@@ -1,5 +1,4 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* vim: set ts=4 sw=4 et tw=78: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -42,20 +41,23 @@
 #define nsXULDocument_h__
 
 #include "nsCOMPtr.h"
+#include "nsXULPrototypeDocument.h"
+#include "nsXULPrototypeCache.h"
+#include "nsTArray.h"
+
 #include "nsXMLDocument.h"
 #include "nsElementMap.h"
 #include "nsForwardReference.h"
 #include "nsIContent.h"
-#include "nsIDOMEventReceiver.h"
+#include "nsIDOMEventTarget.h"
 #include "nsIDOMXULCommandDispatcher.h"
 #include "nsIDOMXULDocument.h"
-#include "nsISupportsArray.h"
 #include "nsCOMArray.h"
 #include "nsIURI.h"
 #include "nsIXULDocument.h"
-#include "nsIXULPrototypeDocument.h"
 #include "nsScriptLoader.h"
 #include "nsIStreamListener.h"
+#include "nsICSSLoaderObserver.h"
 
 class nsIRDFResource;
 class nsIRDFService;
@@ -76,16 +78,14 @@ class nsIXULPrototypeScript;
 struct JSObject;
 struct PRLogModuleInfo;
 
-#include "nsIFastLoadService.h"         // XXXbe temporary?
-
 /**
  * The XUL document class
  */
 class nsXULDocument : public nsXMLDocument,
                       public nsIXULDocument,
-                      public nsIDOMXULDocument2,
-                      public nsIDOMXULDocument_MOZILLA_1_8_BRANCH,
-                      public nsIStreamLoaderObserver
+                      public nsIDOMXULDocument,
+                      public nsIStreamLoaderObserver,
+                      public nsICSSLoaderObserver
 {
 public:
     nsXULDocument();
@@ -97,7 +97,8 @@ public:
 
     // nsIDocument interface
     virtual void Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup);
-    virtual void ResetToURI(nsIURI *aURI, nsILoadGroup* aLoadGroup);
+    virtual void ResetToURI(nsIURI *aURI, nsILoadGroup* aLoadGroup,
+                            nsIPrincipal* aPrincipal);
 
     virtual nsresult StartDocumentLoad(const char* aCommand,
                                        nsIChannel *channel,
@@ -107,52 +108,31 @@ public:
                                        PRBool aReset = PR_TRUE,
                                        nsIContentSink* aSink = nsnull);
 
-    virtual nsIPrincipal* GetPrincipal();
-
-    virtual void SetPrincipal(nsIPrincipal *aPrincipal);
-
     virtual void SetContentType(const nsAString& aContentType);
 
     virtual void EndLoad();
 
-    virtual void ContentAppended(nsIContent* aContainer,
-                                 PRInt32 aNewIndexInContainer);
-
-    virtual void ContentInserted(nsIContent* aContainer,
-                                 nsIContent* aChild,
-                                 PRInt32 aIndexInContainer);
-
-    virtual void ContentRemoved(nsIContent* aContainer,
-                                nsIContent* aChild,
-                                PRInt32 aIndexInContainer);
-
-    virtual void AttributeChanged(nsIContent* aElement, PRInt32 aNameSpaceID,
-                                  nsIAtom* aAttribute, PRInt32 aModType);
-
-    virtual nsresult HandleDOMEvent(nsPresContext* aPresContext,
-                                    nsEvent* aEvent,
-                                    nsIDOMEvent** aDOMEvent,
-                                    PRUint32 aFlags,
-                                    nsEventStatus* aEventStatus);
+    // nsIMutationObserver interface
+    NS_DECL_NSIMUTATIONOBSERVER_CONTENTAPPENDED
+    NS_DECL_NSIMUTATIONOBSERVER_CONTENTINSERTED
+    NS_DECL_NSIMUTATIONOBSERVER_CONTENTREMOVED
+    NS_DECL_NSIMUTATIONOBSERVER_ATTRIBUTECHANGED
 
     // nsIXULDocument interface
     NS_IMETHOD AddElementForID(const nsAString& aID, nsIContent* aElement);
     NS_IMETHOD RemoveElementForID(const nsAString& aID, nsIContent* aElement);
     NS_IMETHOD GetElementsForID(const nsAString& aID,
-                                nsISupportsArray* aElements);
-    NS_IMETHOD AddForwardReference(nsForwardReference* aRef);
-    NS_IMETHOD ResolveForwardReferences();
-    NS_IMETHOD SetMasterPrototype(nsIXULPrototypeDocument* aDocument);
-    NS_IMETHOD GetMasterPrototype(nsIXULPrototypeDocument** aDocument);
-    NS_IMETHOD SetCurrentPrototype(nsIXULPrototypeDocument* aDocument);
+                                nsCOMArray<nsIContent>& aElements);
+
+    NS_IMETHOD GetScriptGlobalObjectOwner(nsIScriptGlobalObjectOwner** aGlobalOwner);
     NS_IMETHOD AddSubtreeToDocument(nsIContent* aElement);
     NS_IMETHOD RemoveSubtreeFromDocument(nsIContent* aElement);
     NS_IMETHOD SetTemplateBuilderFor(nsIContent* aContent,
                                      nsIXULTemplateBuilder* aBuilder);
     NS_IMETHOD GetTemplateBuilderFor(nsIContent* aContent,
                                      nsIXULTemplateBuilder** aResult);
-    NS_IMETHOD OnPrototypeLoadDone();
-    NS_IMETHOD OnHide();
+    NS_IMETHOD OnPrototypeLoadDone(PRBool aResumeWalk);
+    PRBool OnDocumentParserError();
 
     // nsIDOMNode interface overrides
     NS_IMETHOD CloneNode(PRBool deep, nsIDOMNode **_retval);
@@ -163,17 +143,25 @@ public:
 
     // nsIDOMXULDocument interface
     NS_DECL_NSIDOMXULDOCUMENT
-    NS_DECL_NSIDOMXULDOCUMENT2
-    NS_DECL_NSIDOMXULDOCUMENT_MOZILLA_1_8_BRANCH
 
     // nsIDOMNSDocument
     NS_IMETHOD GetContentType(nsAString& aContentType);
 
+    // nsICSSLoaderObserver
+    NS_IMETHOD StyleSheetLoaded(nsICSSStyleSheet* aSheet,
+                                PRBool aWasAlternate,
+                                nsresult aStatus);
+
+    virtual void EndUpdate(nsUpdateType aUpdateType);
+
     static PRBool
     MatchAttribute(nsIContent* aContent,
                    PRInt32 aNameSpaceID,
                    nsIAtom* aAttrName,
-                   const nsAString& aValue);
+                   void* aData);
+
+    NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED_NO_UNLINK(nsXULDocument,
+                                                       nsXMLDocument)
 
 protected:
     // Implementation methods
@@ -189,15 +177,14 @@ protected:
     nsresult
     RemoveElementFromMap(nsIContent* aElement);
 
-    nsresult GetPixelDimensions(nsIPresShell* aShell, PRInt32* aWidth,
-                                PRInt32* aHeight);
+    nsresult GetViewportSize(PRInt32* aWidth, PRInt32* aHeight);
 
     static PRIntn
     RemoveElementsFromMapByContent(const PRUnichar* aID,
                                    nsIContent* aElement,
                                    void* aClosure);
 
-    void SetIsPopup(PRBool isPopup) { mIsPopup = isPopup; };
+    void SetIsPopup(PRBool isPopup) { mIsPopup = isPopup; }
 
     nsresult PrepareToLoad(nsISupports* aContainer,
                            const char* aCommand,
@@ -212,11 +199,13 @@ protected:
                            nsIParser** aResult);
 
     nsresult 
-    LoadOverlayInternal(nsIURI* aURI, PRBool aIsDynamic, PRBool* aShouldReturn);
+    LoadOverlayInternal(nsIURI* aURI, PRBool aIsDynamic, PRBool* aShouldReturn,
+                        PRBool* aFailureFromContent);
 
     nsresult ApplyPersistentAttributes();
+    nsresult ApplyPersistentAttributesInternal();
     nsresult ApplyPersistentAttributesToElements(nsIRDFResource* aResource,
-                                                 nsISupportsArray* aElements);
+                                                 nsCOMArray<nsIContent>& aElements);
 
     nsresult
     AddElementToDocumentPre(nsIContent* aElement);
@@ -234,7 +223,7 @@ protected:
     PRInt32 GetDefaultNamespaceID() const
     {
         return kNameSpaceID_XUL;
-    };
+    }
 
 protected:
     // pseudo constants
@@ -247,15 +236,15 @@ protected:
     static nsIRDFResource* kNC_attribute;
     static nsIRDFResource* kNC_value;
 
-    static nsIXULPrototypeCache* gXULCache;
+    static nsXULPrototypeCache* gXULCache;
 
     static PRLogModuleInfo* gXULLog;
 
-    nsresult
-    Persist(nsIContent* aElement, PRInt32 aNameSpaceID, nsIAtom* aAttribute);
+    PRBool
+    IsCapabilityEnabled(const char* aCapabilityLabel);
 
     nsresult
-    DestroyForwardReferences();
+    Persist(nsIContent* aElement, PRInt32 aNameSpaceID, nsIAtom* aAttribute);
 
     // IMPORTANT: The ownership implicit in the following member
     // variables has been explicitly checked and set using nsCOMPtr
@@ -270,18 +259,33 @@ protected:
     nsElementMap               mElementMap;
     nsCOMPtr<nsIRDFDataSource> mLocalStore;
     PRPackedBool               mIsPopup;
-    PRPackedBool               mIsFastLoad;
     PRPackedBool               mApplyingPersistedAttrs;
     PRPackedBool               mIsWritingFastLoad;
     PRPackedBool               mDocumentLoaded;
+    /**
+     * Since ResumeWalk is interruptible, it's possible that last
+     * stylesheet finishes loading while the PD walk is still in
+     * progress (waiting for an overlay to finish loading).
+     * mStillWalking prevents DoneLoading (and StartLayout) from being
+     * called in this situation.
+     */
+    PRPackedBool               mStillWalking;
+
+    /**
+     * An array of style sheets, that will be added (preserving order) to the
+     * document after all of them are loaded (in DoneWalking).
+     */
+    nsCOMArray<nsICSSStyleSheet> mOverlaySheets;
+
     nsCOMPtr<nsIDOMXULCommandDispatcher>     mCommandDispatcher; // [OWNER] of the focus tracker
 
     // Maintains the template builders that have been attached to
     // content elements
-    nsSupportsHashtable* mTemplateBuilderTable;
-    
-    nsVoidArray mForwardReferences;
-    nsForwardReference::Phase mResolutionPhase;
+    typedef nsInterfaceHashtable<nsISupportsHashKey, nsIXULTemplateBuilder>
+        BuilderTable;
+    BuilderTable* mTemplateBuilderTable;
+
+    PRUint32 mPendingSheets;
 
     /*
      * XXX dr
@@ -344,8 +348,13 @@ protected:
      * order of the array is significant: overlays at the _end_ of the
      * array are resolved before overlays earlier in the array (i.e.,
      * it is a stack).
+     *
+     * In the current implementation the order the overlays are loaded
+     * in is as follows: first overlays from xul-overlay PIs, in the
+     * same order as in the document, then the overlays from the chrome
+     * registry.
      */
-    nsCOMPtr<nsISupportsArray> mUnloadedOverlays;
+    nsCOMArray<nsIURI> mUnloadedOverlays;
 
     /**
      * Load the transcluded script at the specified URI. If the
@@ -358,7 +367,13 @@ protected:
      * Execute the precompiled script object scoped by this XUL document's
      * containing window object, and using its associated script context.
      */
-    nsresult ExecuteScript(JSObject* aScriptObject);
+    nsresult ExecuteScript(nsIScriptContext *aContext, void* aScriptObject);
+
+    /**
+     * Helper method for the above that uses aScript to find the appropriate
+     * script context and object.
+     */
+    nsresult ExecuteScript(nsXULPrototypeScript *aScript);
 
     /**
      * Create a delegate content model element from a prototype.
@@ -368,8 +383,8 @@ protected:
                                         nsIContent** aResult);
 
     /**
-     * Create a temporary 'overlay' element to which content nodes
-     * can be attached for later resolution.
+     * Create a hook-up element to which content nodes can be attached for
+     * later resolution.
      */
     nsresult CreateOverlayElement(nsXULPrototypeElement* aPrototype, nsIContent** aResult);
 
@@ -386,12 +401,6 @@ protected:
      */
     nsXULPrototypeScript* mCurrentScriptProto;
 
-    /**
-     * A "dummy" channel that is used as a placeholder to signal document load
-     * completion.
-     */
-    nsCOMPtr<nsIRequest> mPlaceHolderRequest;
-        
     /**
      * Check if a XUL template builder has already been hooked up.
      */
@@ -405,15 +414,40 @@ protected:
     CreateTemplateBuilder(nsIContent* aElement);
 
     /**
-     * Do hookup for <xul:observes> tag
-     */
-    nsresult HookupObserver(nsIContent* aElement);
-
-    /**
-     * Add the current prototype's style sheets to the document.
+     * Add the current prototype's style sheets (currently it's just
+     * style overlays from the chrome registry) to the document.
      */
     nsresult AddPrototypeSheets();
 
+
+protected:
+    /* Declarations related to forward references. 
+     *
+     * Forward references are declarations which are added to the temporary
+     * list (mForwardReferences) during the document (or overlay) load and
+     * are resolved later, when the document loading is almost complete.
+     */
+
+    /**
+     * The list of different types of forward references to resolve. After
+     * a reference is resolved, it is removed from this array (and
+     * automatically deleted)
+     */
+    nsTArray<nsAutoPtr<nsForwardReference> > mForwardReferences;
+
+    /** Indicates what kind of forward references are still to be processed. */
+    nsForwardReference::Phase mResolutionPhase;
+
+    /**
+     * Adds aRef to the mForwardReferences array. Takes the ownership of aRef.
+     */
+    nsresult AddForwardReference(nsForwardReference* aRef);
+
+    /**
+     * Resolve all of the document's forward references.
+     */
+    nsresult ResolveForwardReferences();
+
     /**
      * Used to resolve broadcaster references
      */
@@ -514,25 +548,58 @@ protected:
      * The current prototype that we are walking to construct the
      * content model.
      */
-    nsCOMPtr<nsIXULPrototypeDocument> mCurrentPrototype;
+    nsRefPtr<nsXULPrototypeDocument> mCurrentPrototype;
 
     /**
      * The master document (outermost, .xul) prototype, from which
      * all subdocuments get their security principals.
      */
-    nsCOMPtr<nsIXULPrototypeDocument> mMasterPrototype;
+    nsRefPtr<nsXULPrototypeDocument> mMasterPrototype;
 
     /**
      * Owning references to all of the prototype documents that were
      * used to construct this document.
      */
-    nsCOMArray<nsIXULPrototypeDocument> mPrototypes;
+    nsTArray< nsRefPtr<nsXULPrototypeDocument> > mPrototypes;
 
     /**
      * Prepare to walk the current prototype.
      */
     nsresult PrepareToWalk();
 
+    /**
+     * Creates a processing instruction based on aProtoPI and inserts
+     * it to the DOM (as the aIndex-th child of aParent).
+     */
+    nsresult
+    CreateAndInsertPI(const nsXULPrototypePI* aProtoPI,
+                      nsINode* aParent, PRUint32 aIndex);
+
+    /**
+     * Inserts the passed <?xml-stylesheet ?> PI at the specified
+     * index. Loads and applies the associated stylesheet
+     * asynchronously.
+     * The prototype document walk can happen before the stylesheets
+     * are loaded, but the final steps in the load process (see
+     * DoneWalking()) are not run before all the stylesheets are done
+     * loading.
+     */
+    nsresult
+    InsertXMLStylesheetPI(const nsXULPrototypePI* aProtoPI,
+                          nsINode* aParent,
+                          PRUint32 aIndex,
+                          nsIContent* aPINode);
+
+    /**
+     * Inserts the passed <?xul-overlay ?> PI at the specified index.
+     * Schedules the referenced overlay URI for further processing.
+     */
+    nsresult
+    InsertXULOverlayPI(const nsXULPrototypePI* aProtoPI,
+                       nsINode* aParent,
+                       PRUint32 aIndex,
+                       nsIContent* aPINode);
+
     /**
      * Add overlays from the chrome registry to the set of unprocessed
      * overlays still to do.
@@ -545,6 +612,13 @@ protected:
      */
     nsresult ResumeWalk();
 
+    /**
+     * Called at the end of ResumeWalk() and from StyleSheetLoaded().
+     * Expects that both the prototype document walk is complete and
+     * all referenced stylesheets finished loading.
+     */
+    nsresult DoneWalking();
+
     /**
      * Report that an overlay failed to load
      * @param aURI the URI of the overlay that failed to load
@@ -577,11 +651,13 @@ protected:
 
     class ParserObserver : public nsIRequestObserver {
     protected:
-        nsXULDocument* mDocument;
+        nsRefPtr<nsXULDocument> mDocument;
+        nsRefPtr<nsXULPrototypeDocument> mPrototype;
         virtual ~ParserObserver();
 
     public:
-        ParserObserver(nsXULDocument* aDocument);
+        ParserObserver(nsXULDocument* aDocument,
+                       nsXULPrototypeDocument* aPrototype);
 
         NS_DECL_ISUPPORTS
         NS_DECL_NSIREQUESTOBSERVER
@@ -598,6 +674,27 @@ protected:
     nsInterfaceHashtable<nsURIHashKey,nsIObserver> mPendingOverlayLoadNotifications;
     
     PRBool mInitialLayoutComplete;
+
+    class nsDelayedBroadcastUpdate
+    {
+    public:
+      nsDelayedBroadcastUpdate(nsIDOMElement* aBroadcaster,
+                               nsIDOMElement* aListener,
+                               const nsAString &aAttr)
+      : mBroadcaster(aBroadcaster), mListener(aListener), mAttr(aAttr) {}
+
+      nsDelayedBroadcastUpdate(const nsDelayedBroadcastUpdate& aOther)
+      : mBroadcaster(aOther.mBroadcaster), mListener(aOther.mListener),
+        mAttr(aOther.mAttr) {}
+
+      nsCOMPtr<nsIDOMElement> mBroadcaster;
+      nsCOMPtr<nsIDOMElement> mListener;
+      nsString                mAttr;
+    };
+
+    nsTArray<nsDelayedBroadcastUpdate> mDelayedBroadcasters;
+
+    void MaybeBroadcast();
 private:
     // helpers
 
