@@ -46,7 +46,6 @@
 #include "nsIDOMKeyEvent.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMXPathResult.h"
-#include "nsIXTFXMLVisualWrapper.h"
 #include "nsIDocument.h"
 #include "nsXFormsModelElement.h"
 #include "nsPIDOMWindow.h"
@@ -55,6 +54,7 @@
 #include "nsIEventStateManager.h"
 #include "nsIContent.h"
 #include "nsIDOM3Node.h"
+#include "nsIDOMAttr.h"
 
 /** This class is used to generate xforms-hint and xforms-help events.*/
 class nsXFormsHintHelpListener : public nsIDOMEventListener {
@@ -105,14 +105,25 @@ nsXFormsHintHelpListener::HandleEvent(nsIDOMEvent* aEvent)
 }
 
 NS_IMETHODIMP
-nsXFormsControlStubBase::GetBoundNode(nsIDOMNode **aBoundNode)
+nsXFormsControlStub::GetModel(nsIModelElementPrivate **aModel)
 {
+  NS_ENSURE_ARG_POINTER(aModel);
+
+  NS_IF_ADDREF(*aModel = mModel);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXFormsControlStub::GetBoundNode(nsIDOMNode **aBoundNode)
+{
+  NS_ENSURE_ARG_POINTER(aBoundNode);
+
   NS_IF_ADDREF(*aBoundNode = mBoundNode);
   return NS_OK;  
 }
 
 NS_IMETHODIMP
-nsXFormsControlStubBase::GetDependencies(nsCOMArray<nsIDOMNode> **aDependencies)
+nsXFormsControlStub::GetDependencies(nsCOMArray<nsIDOMNode> **aDependencies)
 {
   if (aDependencies)
     *aDependencies = &mDependencies;
@@ -120,14 +131,14 @@ nsXFormsControlStubBase::GetDependencies(nsCOMArray<nsIDOMNode> **aDependencies)
 }
 
 NS_IMETHODIMP
-nsXFormsControlStubBase::GetElement(nsIDOMElement **aElement)
+nsXFormsControlStub::GetElement(nsIDOMElement **aElement)
 {
   NS_IF_ADDREF(*aElement = mElement);
   return NS_OK;  
 }
 
 void
-nsXFormsControlStubBase::RemoveIndexListeners()
+nsXFormsControlStub::RemoveIndexListeners()
 {
   if (!mIndexesUsed.Count())
     return;
@@ -141,33 +152,38 @@ nsXFormsControlStubBase::RemoveIndexListeners()
 }
 
 NS_IMETHODIMP
-nsXFormsControlStubBase::ResetBoundNode(const nsString     &aBindAttribute,
-                                        PRUint16            aResultType,
-                                        nsIDOMXPathResult **aResult)
+nsXFormsControlStub::ResetBoundNode(const nsString &aBindAttribute,
+                                    PRUint16        aResultType,
+                                    PRBool         *aContextChanged)
 {
+  NS_ENSURE_ARG(aContextChanged);
+
   // Clear existing bound node, etc.
-  mBoundNode = nsnull;
+  *aContextChanged = mBoundNode ? PR_TRUE : PR_FALSE;
+  nsCOMPtr<nsIDOMNode> oldBoundNode;
+  oldBoundNode.swap(mBoundNode);
   mUsesModelBinding = PR_FALSE;
+  mAppearDisabled = PR_FALSE;
   mDependencies.Clear();
   RemoveIndexListeners();
 
-  if (!mHasParent || !mBindAttrsCount)
-    return NS_OK;
+  if (!mHasParent || !mHasDoc || !HasBindingAttribute())
+    return NS_OK_XFORMS_NOTREADY;
 
   nsCOMPtr<nsIDOMXPathResult> result;
-  nsresult rv =
-    ProcessNodeBinding(aBindAttribute,
-                       aResultType,
-                       getter_AddRefs(result));
+  nsresult rv = ProcessNodeBinding(aBindAttribute, aResultType,
+                                   getter_AddRefs(result));
 
   if (NS_FAILED(rv)) {
     nsXFormsUtils::ReportError(NS_LITERAL_STRING("controlBindError"), mElement);
     return rv;
   }
-  
-  if (!result)
-    return NS_OK;
-    
+
+  if (rv == NS_OK_XFORMS_DEFERRED || rv == NS_OK_XFORMS_NOTREADY || !result) {
+    // Binding was deferred, or not bound
+    return rv;
+  }
+
   // Get context node, if any
   if (mUsesModelBinding) {
     // When bound via @bind, we'll get a snapshot back
@@ -176,100 +192,195 @@ nsXFormsControlStubBase::ResetBoundNode(const nsString     &aBindAttribute,
     result->GetSingleNodeValue(getter_AddRefs(mBoundNode));
   }
 
-  if (mBoundNode && mModel) {
-    mModel->SetStates(this, mBoundNode);
-  } else if (mModel) {
-    // we should have been successful.  Must be pointing to a node that
-    // doesn't exist in the instance document.  Disable the control
-    // per 4.2.2 in the spec
+  *aContextChanged = (oldBoundNode != mBoundNode);
 
-    nsCOMPtr<nsIXTFElementWrapper> xtfWrap(do_QueryInterface(mElement));
-    NS_ENSURE_STATE(xtfWrap);
-    xtfWrap->SetIntrinsicState(NS_EVENT_STATE_DISABLED);
+  // Some controls may not be bound to certain types of content. If the content
+  // is a disallowed type, report the error and dispatch a binding exception
+  // event.
+  PRBool isAllowed = IsContentAllowed();
 
-    // Dispatch event
-    nsXFormsUtils::DispatchEvent(mElement, eEvent_Disabled);
+  if (!mBoundNode || !isAllowed) {
+    // If there's no result (ie, no instance node) returned by the above, it
+    // means that the binding is not pointing to an instance data node, so we
+    // should disable the control.
+    mAppearDisabled = PR_TRUE;
+
+    if (!isAllowed) {
+      // build the error string that we want output to the ErrorConsole
+      nsAutoString localName;
+      mElement->GetLocalName(localName);
+      const PRUnichar *strings[] = { localName.get() };
+
+      nsXFormsUtils::ReportError(
+        NS_LITERAL_STRING("boundTypeErrorComplexContent"),
+        strings, 1, mElement, mElement);
+
+      nsXFormsUtils::DispatchEvent(mElement, eEvent_BindingException);
+    }
+
+    nsCOMPtr<nsIXTFElementWrapper> wrapper(do_QueryInterface(mElement));
+    NS_ENSURE_STATE(wrapper);
+
+    PRInt32 iState;
+    GetDisabledIntrinsicState(&iState);
+    return wrapper->SetIntrinsicState(iState);
   }
 
-  if (aResult) {
-    *aResult = nsnull;
-    result.swap(*aResult);
+  // Check for presence of @xsi:type on bound node and add as a dependency
+  nsCOMPtr<nsIDOMElement> boundEl(do_QueryInterface(mBoundNode));
+  if (boundEl) {
+    nsCOMPtr<nsIDOMAttr> attrNode;
+    rv = boundEl->GetAttributeNodeNS(NS_LITERAL_STRING(NS_NAMESPACE_XML_SCHEMA_INSTANCE),
+                                     NS_LITERAL_STRING("type"),
+                                     getter_AddRefs(attrNode));
+    if (NS_SUCCEEDED(rv) && attrNode) {
+      mDependencies.AppendObject(attrNode);
+    }
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXFormsControlStubBase::Bind()
+nsXFormsControlStub::Bind(PRBool* aContextChanged)
 {
   return ResetBoundNode(NS_LITERAL_STRING("ref"),
-                        nsIDOMXPathResult::FIRST_ORDERED_NODE_TYPE);
+                        nsIDOMXPathResult::FIRST_ORDERED_NODE_TYPE,
+                        aContextChanged);
 }
 
 NS_IMETHODIMP
-nsXFormsControlStubBase::TryFocus(PRBool* aOK)
+nsXFormsControlStub::Refresh()
+{
+  // XXX: In theory refresh should never be called when there is no model,
+  // but that's definately not the case now.
+  return (mModel && !mAppearDisabled) ? mModel->SetStates(this, mBoundNode)
+                                      : NS_OK;
+}
+
+NS_IMETHODIMP
+nsXFormsControlStub::TryFocus(PRBool* aOK)
 {
   *aOK = PR_FALSE;
   return NS_OK;
 }
   
 NS_IMETHODIMP
-nsXFormsControlStubBase::IsEventTarget(PRBool *aOK)
+nsXFormsControlStub::IsEventTarget(PRInt32 aEvent, PRBool *aOK)
 {
+  NS_ENSURE_ARG_POINTER(aOK);
   *aOK = PR_TRUE;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXFormsControlStubBase::GetUsesModelBinding(PRBool *aRes)
+nsXFormsControlStub::GetUsesModelBinding(PRBool *aRes)
 {
   *aRes = mUsesModelBinding;
   return NS_OK;
 }
 
-nsresult
-nsXFormsControlStubBase::MaybeAddToModel(nsIModelElementPrivate *aOldModel,
-                                         nsIXFormsControl       *aParent)
+NS_IMETHODIMP
+nsXFormsControlStub::GetUsesSingleNodeBinding(PRBool *aRes)
 {
+  *aRes = PR_TRUE;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXFormsControlStub::GetOnDeferredBindList(PRBool *aOnList)
+{
+  NS_ENSURE_ARG_POINTER(aOnList);
+  *aOnList = mOnDeferredBindList;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXFormsControlStub::SetOnDeferredBindList(PRBool aPutOnList)
+{
+  mOnDeferredBindList = aPutOnList;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXFormsControlStub::GetDefaultIntrinsicState(PRInt32 *aState)
+{
+  NS_ENSURE_ARG_POINTER(aState);
+  *aState = kDefaultIntrinsicState;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXFormsControlStub::GetDisabledIntrinsicState(PRInt32 *aState)
+{
+  NS_ENSURE_ARG_POINTER(aState);
+  *aState = kDisabledIntrinsicState;
+  return NS_OK;
+}
+
+nsresult
+nsXFormsControlStub::MaybeAddToModel(nsIModelElementPrivate *aOldModel,
+                                     nsIXFormsControl       *aParent)
+{
+  if (GetRepeatState() == eType_Template) {
+    // No sense going any further.  A template control has no need to bind,
+    // refresh, etc. so no reason to put it on the model's control list
+    return NS_OK;
+  }
+
   // XXX: just doing pointer comparison would be nice....
   PRBool sameModel = PR_FALSE;
-  nsCOMPtr<nsIDOM3Node> n3Model(do_QueryInterface(mModel));
-  nsCOMPtr<nsIDOMNode> nOldModel(do_QueryInterface(aOldModel));
-  NS_ASSERTION(n3Model, "model element not supporting nsIDOM3Node?!");
-  nsresult rv = n3Model->IsSameNode(nOldModel, &sameModel);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv;
+
+  if (mModel) {
+    nsCOMPtr<nsIDOM3Node> n3Model(do_QueryInterface(mModel));
+    nsCOMPtr<nsIDOMNode> nOldModel(do_QueryInterface(aOldModel));
+    NS_ASSERTION(n3Model, "model element not supporting nsIDOM3Node?!");
+    rv = n3Model->IsSameNode(nOldModel, &sameModel);
+    NS_ENSURE_SUCCESS(rv, rv);
+  } else {
+    sameModel = !aOldModel;
+  }
+
   if (!sameModel) {
     if (aOldModel) {
       rv = aOldModel->RemoveFormControl(this);
       NS_ENSURE_SUCCESS(rv, rv);
     }
-    rv = mModel->AddFormControl(this, aParent);
-    NS_ENSURE_SUCCESS(rv, rv);
+    if (mModel) {
+      rv = mModel->AddFormControl(this, aParent);
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
   }
   return NS_OK;
 }
 
 
 nsresult
-nsXFormsControlStubBase::ProcessNodeBinding(const nsString          &aBindingAttr,
-                                            PRUint16                 aResultType,
-                                            nsIDOMXPathResult      **aResult,
-                                            nsIModelElementPrivate **aModel)
+nsXFormsControlStub::ProcessNodeBinding(const nsString          &aBindingAttr,
+                                        PRUint16                 aResultType,
+                                        nsIDOMXPathResult      **aResult,
+                                        nsIModelElementPrivate **aModel)
 {
   nsStringArray indexesUsed;
 
+  if (aResult) {
+    *aResult = nsnull;
+  }
+
+  if (aModel) {
+    *aModel = nsnull;
+  }
+
   // let's not go through all of this rigamarol if we don't have a chance
   // in heck of binding anyhow.  Check to see if the models will be receptive
   // to some binding.  readyForBindProperty is set when they are.  Make sure
   // to return NS_OK so that we don't start complaining about binding
   // failures in this situation.
 
-  nsCOMPtr<nsIDOMDocument> domDoc;
-  mElement->GetOwnerDocument(getter_AddRefs(domDoc));
-
-  if (!nsXFormsUtils::IsDocumentReadyForBind(domDoc)) {
-    nsXFormsModelElement::DeferElementBind(domDoc, this);
-    return NS_OK;
+  if (!nsXFormsUtils::IsDocumentReadyForBind(mElement)) {
+    nsXFormsModelElement::DeferElementBind(this);
+    return NS_OK_XFORMS_DEFERRED;
   }
 
   nsresult rv;
@@ -287,6 +398,14 @@ nsXFormsControlStubBase::ProcessNodeBinding(const nsString          &aBindingAtt
                                           getter_AddRefs(parentControl),
                                           &mDependencies,
                                           &indexesUsed);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // if NS_OK_XFORMS_NOTREADY, mModel probably won't exist or might no longer
+  // be valid, so shouldn't continue.
+  if (rv == NS_OK_XFORMS_NOTREADY) {
+    return rv;
+  }
+
   NS_ENSURE_STATE(mModel);
 
   rv = MaybeAddToModel(oldModel, parentControl);
@@ -294,34 +413,41 @@ nsXFormsControlStubBase::ProcessNodeBinding(const nsString          &aBindingAtt
 
   if (aModel)
     NS_ADDREF(*aModel = mModel);
+
   mUsesModelBinding = usesModelBinding;
 
-  if (NS_SUCCEEDED(rv) && indexesUsed.Count()) {
+  if (indexesUsed.Count()) {
     // add index listeners on repeat elements
 
     for (PRInt32 i = 0; i < indexesUsed.Count(); ++i) {
       // Find the repeat element and add |this| as a listener
       nsCOMPtr<nsIDOMElement> repElem;
-      domDoc->GetElementById(*(indexesUsed[i]), getter_AddRefs(repElem));
+      nsXFormsUtils::GetElementByContextId(mElement, *(indexesUsed[i]),
+                                           getter_AddRefs(repElem));
       nsCOMPtr<nsIXFormsRepeatElement> rep(do_QueryInterface(repElem));
       if (!rep)
         continue;
 
       rv = rep->AddIndexUser(this);
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
+      NS_ENSURE_SUCCESS(rv, rv);
+
       rv = mIndexesUsed.AppendObject(rep);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
 
-  return rv;
+  return NS_OK;
 }
 
-nsresult
-nsXFormsControlStubBase::BindToModel(PRBool aSetBoundNode)
+NS_IMETHODIMP
+nsXFormsControlStub::BindToModel(PRBool aSetBoundNode)
 {
+  if (GetRepeatState() == eType_Template) {
+    // No sense going any further.  A template control has no need to bind,
+    // refresh, etc. so no reason to put it on the model's control list
+    return NS_OK;
+  }
+
   nsCOMPtr<nsIModelElementPrivate> oldModel(mModel);
 
   nsCOMPtr<nsIXFormsControl> parentControl;
@@ -337,7 +463,7 @@ nsXFormsControlStubBase::BindToModel(PRBool aSetBoundNode)
 }
 
 void
-nsXFormsControlStubBase::ResetHelpAndHint(PRBool aInitialize)
+nsXFormsControlStub::ResetHelpAndHint(PRBool aInitialize)
 {
   nsCOMPtr<nsIDOMEventTarget> targ(do_QueryInterface(mElement));
   if (!targ)
@@ -372,7 +498,7 @@ nsXFormsControlStubBase::ResetHelpAndHint(PRBool aInitialize)
 }
 
 PRBool
-nsXFormsControlStubBase::GetRelevantState()
+nsXFormsControlStub::GetRelevantState()
 {
   PRBool res = PR_FALSE;
   nsCOMPtr<nsIContent> content(do_QueryInterface(mElement));
@@ -382,9 +508,9 @@ nsXFormsControlStubBase::GetRelevantState()
   return res;
 }
 
-nsresult
-nsXFormsControlStubBase::HandleDefault(nsIDOMEvent *aEvent,
-                                       PRBool      *aHandled)
+NS_IMETHODIMP
+nsXFormsControlStub::HandleDefault(nsIDOMEvent *aEvent,
+                                   PRBool      *aHandled)
 {
   NS_ENSURE_ARG(aHandled);
   *aHandled = PR_FALSE;
@@ -403,7 +529,7 @@ nsXFormsControlStubBase::HandleDefault(nsIDOMEvent *aEvent,
     nsAutoString type;
     aEvent->GetType(type);
 
-    if (type.EqualsASCII(sXFormsEventsEntries[eEvent_Focus].name)) {
+    if (type.EqualsLiteral(sXFormsEventsEntries[eEvent_Focus].name)) {
       TryFocus(aHandled);
     } else if (type.Equals(NS_LITERAL_STRING("keypress"))) { 
       nsCOMPtr<nsIDOMKeyEvent> keyEvent = do_QueryInterface(aEvent);
@@ -437,8 +563,8 @@ nsXFormsControlStubBase::HandleDefault(nsIDOMEvent *aEvent,
           }
         }
       }
-    } else if (type.EqualsASCII(sXFormsEventsEntries[eEvent_Next].name) ||     
-               type.EqualsASCII(sXFormsEventsEntries[eEvent_Previous].name)) { 
+    } else if (type.EqualsLiteral(sXFormsEventsEntries[eEvent_Next].name) ||
+               type.EqualsLiteral(sXFormsEventsEntries[eEvent_Previous].name)) {
 
       // only continue this processing if xforms-next or xforms-previous were
       // dispatched by the form and not as part of the 'tab' and 'shift+tab'
@@ -463,14 +589,15 @@ nsXFormsControlStubBase::HandleDefault(nsIDOMEvent *aEvent,
       nsIFocusController *focusController =
         doc->GetWindow()->GetRootFocusController();
       if (focusController &&
-          type.EqualsASCII(sXFormsEventsEntries[eEvent_Next].name)) {
+          type.EqualsLiteral(sXFormsEventsEntries[eEvent_Next].name)) {
         focusController->MoveFocus(PR_TRUE, nsnull);
       } else {
         focusController->MoveFocus(PR_FALSE, nsnull);
       }
-    } else if (type.EqualsASCII(sXFormsEventsEntries[eEvent_BindingException].name)) {
-      *aHandled = nsXFormsUtils::HandleFatalError(mElement,
-                                                  NS_LITERAL_STRING("XFormsBindingException"));
+    } else if (type.EqualsLiteral(sXFormsEventsEntries[eEvent_BindingException].name)) {
+      // we threw up a popup during the nsXFormsUtils::DispatchEvent that sent
+      // this error to this control
+      *aHandled = PR_TRUE;
     }
   }
   
@@ -489,8 +616,8 @@ public:
 
   ~ControlDebug() {
     for (PRInt32 i = 0; i < sControlList->Count(); ++i) {
-      nsXFormsControlStubBase* control =
-        NS_STATIC_CAST(nsXFormsControlStubBase*, sControlList->ElementAt(i));
+      nsXFormsControlStub* control =
+        static_cast<nsXFormsControlStub*>(sControlList->ElementAt(i));
       if (control) {
         printf("Possible leak, <xforms:%s>\n", control->Name());
       }
@@ -503,9 +630,10 @@ public:
 static ControlDebug tester = ControlDebug();
 #endif
 
-nsresult
-nsXFormsControlStubBase::Create(nsIXTFElementWrapper *aWrapper)
+NS_IMETHODIMP
+nsXFormsControlStub::OnCreated(nsIXTFElementWrapper *aWrapper)
 {
+  nsXFormsStubElement::OnCreated(aWrapper);
   aWrapper->SetNotificationMask(kStandardNotificationMask);
 
   // It's ok to keep a weak pointer to mElement.  mElement will have an
@@ -516,8 +644,6 @@ nsXFormsControlStubBase::Create(nsIXTFElementWrapper *aWrapper)
   mElement = node;
   NS_ASSERTION(mElement, "Wrapper is not an nsIDOMElement, we'll crash soon");
 
-  ResetHelpAndHint(PR_TRUE);
-
 #ifdef DEBUG_smaug
   sControlList->AppendElement(this);
 #endif
@@ -525,10 +651,9 @@ nsXFormsControlStubBase::Create(nsIXTFElementWrapper *aWrapper)
   return NS_OK;
 }
 
-nsresult
-nsXFormsControlStubBase::OnDestroyed()
+NS_IMETHODIMP
+nsXFormsControlStub::OnDestroyed()
 {
-  ResetHelpAndHint(PR_FALSE);
   RemoveIndexListeners();
   mDependencies.Clear();
 
@@ -537,6 +662,8 @@ nsXFormsControlStubBase::OnDestroyed()
     mModel = nsnull;
   }
 
+  mAbortedBindListContainer = nsnull;
+
 #ifdef DEBUG_smaug
   sControlList->RemoveElement(this);
 #endif
@@ -546,100 +673,203 @@ nsXFormsControlStubBase::OnDestroyed()
 }
 
 nsresult
-nsXFormsControlStubBase::ForceModelRebind()
+nsXFormsControlStub::ForceModelDetach(PRBool aRebind)
 {
+  // We shouldn't bother binding if the control is part of a template, but we
+  // need to run through RemoveFormControl in the event that this control
+  // was just moved under a repeat or itemset and wasn't there originally.
   if (mModel) {
     // Remove from model, so Bind() will be forced to reattach
     mModel->RemoveFormControl(this);
     mModel = nsnull;
   }
-  nsresult rv = Bind();
+
+  if (!aRebind || GetRepeatState() == eType_Template) {
+    return NS_OK;
+  }
+
+  PRBool dummy;
+  nsresult rv = Bind(&dummy);
   NS_ENSURE_SUCCESS(rv, rv);
+
+  if (rv == NS_OK_XFORMS_DEFERRED || rv == NS_OK_XFORMS_NOTREADY) {
+    return NS_OK;
+  }
+
+  // If there were any controls that had already tried to bind but failed
+  // to because this control wasn't ready yet, then try them again.
+  PRInt32 arraySize = mAbortedBindList.Count();
+  if (arraySize) {
+    for (PRInt32 i = 0; i < arraySize; ++i) {
+      nsCOMPtr<nsIXFormsControl> control = mAbortedBindList.ObjectAt(i);
+      if (control) {
+        control->RebindAndRefresh();
+        control->SetAbortedBindListContainer(nsnull);
+      }
+    }
+  
+    mAbortedBindList.Clear();
+  }
+
   return Refresh();
 }
 
-
-nsresult
-nsXFormsControlStubBase::DocumentChanged(nsIDOMDocument *aNewDocument)
+NS_IMETHODIMP
+nsXFormsControlStub::WillChangeDocument(nsIDOMDocument *aNewDocument)
 {
-  // We need to re-evaluate our instance data binding when our document
-  // changes, since our context can change
-
-  if (aNewDocument) {
-    // The intrinsic state needs to be initialized here since
-    // SetIntrinsicState will do nothing until the element lives in a document.
-    ResetProperties();
-  
-    return ForceModelRebind();
+  // This control is moving to another document or just plain getting removed
+  // from its current document.  In either case, we know that we don't care
+  // about the bind-ready notification we might have been waiting on.
+  nsCOMPtr<nsIXFormsContextControl> ctxtControl;
+  GetAbortedBindListContainer(getter_AddRefs(ctxtControl));
+  if (ctxtControl) {
+    ctxtControl->AddRemoveAbortedControl(this, PR_FALSE);
   }
 
+  // If we are in the middle of getting set up in the current document then
+  // then we need to make sure that if other controls getting set up in
+  // the current document are depending on this control to help them, that
+  // we remove that dependency.
+  PRInt32 arraySize = mAbortedBindList.Count();
+  if (arraySize) {
+    for (PRInt32 i = 0; i < arraySize; ++i) {
+      nsCOMPtr<nsIXFormsControl> control = mAbortedBindList.ObjectAt(i);
+      control->SetAbortedBindListContainer(nsnull);
+    }
+
+    mAbortedBindList.Clear();
+  }
+
+  SetRepeatState(eType_Unknown);
+  ResetHelpAndHint(PR_FALSE);
   return NS_OK;
 }
 
-nsresult
-nsXFormsControlStubBase::ParentChanged(nsIDOMElement *aNewParent)
+NS_IMETHODIMP
+nsXFormsControlStub::DocumentChanged(nsIDOMDocument *aNewDocument)
 {
-  mHasParent = aNewParent != nsnull;
-  // We need to re-evaluate our instance data binding when our parent changes,
-  // since xmlns declarations or our context could have changed.
-  return mHasParent ? ForceModelRebind() : NS_OK;
+  nsXFormsStubElement::DocumentChanged(aNewDocument);
+
+  if (aNewDocument) {
+    ResetHelpAndHint(PR_TRUE);
+
+    // If we are inserted into a document and we have no model, we are probably
+    // being initialized, so we should set our intrinsic state to the default
+    // value
+    if (!mModel && mElement) {
+      nsCOMPtr<nsIXTFElementWrapper> xtfWrap(do_QueryInterface(mElement));
+      NS_ENSURE_STATE(xtfWrap);
+      PRInt32 iState;
+      GetDefaultIntrinsicState(&iState);
+      xtfWrap->SetIntrinsicState(iState);
+    }
+  }
+
+  nsCOMPtr<nsIDOMNode> parent;
+  mElement->GetParentNode(getter_AddRefs(parent));
+  UpdateRepeatState(parent);
+
+  return ForceModelDetach(mHasParent && mHasDoc);
 }
 
-nsresult
-nsXFormsControlStubBase::WillSetAttribute(nsIAtom *aName, const nsAString &aValue)
+NS_IMETHODIMP
+nsXFormsControlStub::WillChangeParent(nsIDOMElement *aNewParent)
+{
+  nsCOMPtr<nsIXFormsContextControl> ctxtControl;
+  GetAbortedBindListContainer(getter_AddRefs(ctxtControl));
+  if (ctxtControl) {
+    ctxtControl->AddRemoveAbortedControl(this, PR_FALSE);
+  }
+
+  SetRepeatState(eType_Unknown);
+  return NS_OK;
+}
+
+
+NS_IMETHODIMP
+nsXFormsControlStub::ParentChanged(nsIDOMElement *aNewParent)
+{
+  nsXFormsStubElement::ParentChanged(aNewParent);
+
+  UpdateRepeatState(aNewParent);
+
+  // We need to re-evaluate our instance data binding when our parent changes,
+  // since xmlns declarations or our context could have changed.
+  return ForceModelDetach(mHasParent && mHasDoc);
+}
+
+NS_IMETHODIMP
+nsXFormsControlStub::WillSetAttribute(nsIAtom *aName, const nsAString &aValue)
 {
   BeforeSetAttribute(aName, aValue);
   return NS_OK;
 }
 
-nsresult
-nsXFormsControlStubBase::AttributeSet(nsIAtom *aName, const nsAString &aValue)
+NS_IMETHODIMP
+nsXFormsControlStub::AttributeSet(nsIAtom *aName, const nsAString &aValue)
 {
   AfterSetAttribute(aName);
   return NS_OK;
 }
 
-nsresult
-nsXFormsControlStubBase::WillRemoveAttribute(nsIAtom *aName)
+NS_IMETHODIMP
+nsXFormsControlStub::WillRemoveAttribute(nsIAtom *aName)
 {
   BeforeSetAttribute(aName, EmptyString());
   return NS_OK;
 }
 
-nsresult
-nsXFormsControlStubBase::AttributeRemoved(nsIAtom *aName)
+NS_IMETHODIMP
+nsXFormsControlStub::AttributeRemoved(nsIAtom *aName)
 {
   AfterSetAttribute(aName);
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsXFormsControlStub::RebindAndRefresh()
+{
+  ForceModelDetach(PR_TRUE);
+  return NS_OK;
+}
+
 // nsIXFormsContextControl
 
 NS_IMETHODIMP
-nsXFormsControlStubBase::SetContext(nsIDOMNode *aContextNode,
-                                    PRInt32     aContextPosition,
-                                    PRInt32     aContextSize)
+nsXFormsControlStub::SetContext(nsIDOMNode *aContextNode,
+                                PRInt32     aContextPosition,
+                                PRInt32     aContextSize)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsXFormsControlStubBase::GetContext(nsAString      &aModelID,
-                                    nsIDOMNode    **aContextNode,
-                                    PRInt32        *aContextPosition,
-                                    PRInt32        *aContextSize)
+nsXFormsControlStub::GetContext(nsAString      &aModelID,
+                                nsIDOMNode    **aContextNode,
+                                PRInt32        *aContextPosition,
+                                PRInt32        *aContextSize)
 {
   NS_ENSURE_ARG(aContextSize);
   NS_ENSURE_ARG(aContextPosition);
+  NS_ENSURE_ARG_POINTER(aContextNode);
 
   *aContextPosition = 1;
   *aContextSize = 1;
+  *aContextNode = nsnull;
 
   if (aContextNode) {
     if (mBoundNode) {
       // We are bound to a node: This is the context node
       NS_ADDREF(*aContextNode = mBoundNode);
     } else if (mModel) {
+      // If there is no bound node, it could be because everything isn't ready,
+      // yet.  If mHasDoc or mHasParent are false, then we haven't had a chance
+      // to bind.  This could happen if the nodes in this chain are being added
+      // via DOM manipulation rather than via the parser.
+      if (HasBindingAttribute() && (!mHasDoc || !mHasParent)) {
+        return NS_OK_XFORMS_NOTREADY;
+      }
+
       // We are bound to a model: The document element of its default instance
       // document is the context node
       nsCOMPtr<nsIDOMDocument> instanceDoc;
@@ -650,6 +880,10 @@ nsXFormsControlStubBase::GetContext(nsAString      &aModelID,
       instanceDoc->GetDocumentElement(&docElement); // addrefs
       NS_ENSURE_STATE(docElement);
       *aContextNode = docElement; // addref'ed above
+    } else {
+      if (HasBindingAttribute() && (!mHasDoc || !mHasParent)) {
+        return NS_OK_XFORMS_NOTREADY;
+      }
     }
   }
 
@@ -663,20 +897,54 @@ nsXFormsControlStubBase::GetContext(nsAString      &aModelID,
   return NS_OK;
 }
 
-void
-nsXFormsControlStubBase::ResetProperties()
+NS_IMETHODIMP
+nsXFormsControlStub::AddRemoveAbortedControl(nsIXFormsControl *aControl,
+                                             PRBool            aAdd)
 {
-  nsCOMPtr<nsIXTFElementWrapper> xtfWrap(do_QueryInterface(mElement));
-  if (!xtfWrap) {
-    return;
+  nsresult rv = NS_OK;
+  if (aAdd) {
+    rv = mAbortedBindList.AppendObject(aControl);
+    aControl->SetAbortedBindListContainer(this);
+  } else {
+    rv = mAbortedBindList.RemoveObject(aControl);
+    aControl->SetAbortedBindListContainer(nsnull);
+  }
+  return rv;
+}
+
+NS_IMETHODIMP
+nsXFormsControlStub::GetAbortedBindListContainer(
+  nsIXFormsContextControl **aControlContainingList)
+{
+  NS_ENSURE_ARG_POINTER(aControlContainingList);
+
+  NS_IF_ADDREF(*aControlContainingList = mAbortedBindListContainer);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXFormsControlStub::SetAbortedBindListContainer(
+  nsIXFormsContextControl *aControlContainingList)
+{
+  // A control should never be on more than one aborted bind list.  There is
+  // probably a chance through some crazy DOM stirring for a control to be
+  // waiting on notification from a context control and then the control finds
+  // itself trying to bind again under a whole different parent tree.  So we'll
+  // have the control remove itself from the old list and link itself to the
+  // new list.
+
+  if (mAbortedBindListContainer && aControlContainingList) {
+    if (!SameCOMIdentity(mAbortedBindListContainer, aControlContainingList)) {
+      mAbortedBindListContainer->AddRemoveAbortedControl(this, PR_FALSE);
+    }
   }
 
-  // enabled is on pr. default
-  xtfWrap->SetIntrinsicState(NS_EVENT_STATE_ENABLED);
+  mAbortedBindListContainer = aControlContainingList;
+  return NS_OK;
 }
 
 void
-nsXFormsControlStubBase::AddRemoveSNBAttr(nsIAtom *aName, const nsAString &aValue) 
+nsXFormsControlStub::AddRemoveSNBAttr(nsIAtom *aName, const nsAString &aValue) 
 {
   nsAutoString attrStr, attrValue;
   aName->ToString(attrStr);
@@ -691,14 +959,11 @@ nsXFormsControlStubBase::AddRemoveSNBAttr(nsIAtom *aName, const nsAString &aValu
     // empty value, treat it like the binding attr is being removed.
     --mBindAttrsCount;
     NS_ASSERTION(mBindAttrsCount>=0, "bad mojo!  mBindAttrsCount < 0!");
-    if (!mBindAttrsCount) {
-      ResetProperties();
-    }
   }
 }
 
 PRBool
-nsXFormsControlStubBase::IsBindingAttribute(const nsIAtom *aAttr) const
+nsXFormsControlStub::IsBindingAttribute(const nsIAtom *aAttr) const
 {
   if (aAttr == nsXFormsAtoms::bind ||
       aAttr == nsXFormsAtoms::ref  ||
@@ -710,33 +975,76 @@ nsXFormsControlStubBase::IsBindingAttribute(const nsIAtom *aAttr) const
 }
 
 void
-nsXFormsControlStubBase::AfterSetAttribute(nsIAtom *aName)
+nsXFormsControlStub::AfterSetAttribute(nsIAtom *aName)
 {
   if (IsBindingAttribute(aName)) {
-    Bind();
-    Refresh();
+    PRBool dummy;
+    nsresult rv = Bind(&dummy);
+    if (NS_SUCCEEDED(rv) &&
+        rv != NS_OK_XFORMS_DEFERRED && rv != NS_OK_XFORMS_NOTREADY)
+      Refresh();
   }
 
 }
 
 void
-nsXFormsControlStubBase::BeforeSetAttribute(nsIAtom         *aName,
-                                            const nsAString &aValue)
+nsXFormsControlStub::BeforeSetAttribute(nsIAtom         *aName,
+                                        const nsAString &aValue)
 {
   if (IsBindingAttribute(aName)) {
     AddRemoveSNBAttr(aName, aValue);
   }
 }
 
+nsresult
+nsXFormsControlStub::GetBoundBuiltinType(PRUint16 *aBuiltinType)
+{
+  NS_ENSURE_ARG_POINTER(aBuiltinType);
+  *aBuiltinType = 0;
+
+  NS_ENSURE_STATE(mModel);
+
+  // get type of bound instance data
+  nsCOMPtr<nsISVSchemaType> schemaType;
+  nsresult rv = mModel->GetTypeForControl(this, getter_AddRefs(schemaType));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return mModel->GetRootBuiltinType(schemaType, aBuiltinType);
+}
+
+PRBool
+nsXFormsControlStub::IsContentAllowed()
+{
+  return PR_TRUE;
+}
+
+PRBool
+nsXFormsControlStub::IsContentComplex()
+{
+  PRBool isComplex = PR_FALSE;
+
+  if (mBoundNode) {
+    // If the bound node has any Element children, then it has complexContent.
+    nsCOMPtr<nsIDOMNode> currentNode;
+    mBoundNode->GetFirstChild(getter_AddRefs(currentNode));
+    while (currentNode) {
+      PRUint16 nodeType;
+      currentNode->GetNodeType(&nodeType);
+      if (nodeType == nsIDOMNode::ELEMENT_NODE) {
+        isComplex = PR_TRUE;
+        break;
+      }
+
+      nsCOMPtr<nsIDOMNode> node;
+      currentNode->GetNextSibling(getter_AddRefs(node));
+      currentNode.swap(node);
+    }
+  }
+  return isComplex;
+}
+
 NS_IMPL_ISUPPORTS_INHERITED3(nsXFormsControlStub,
-                             nsXFormsXMLVisualStub,
-                             nsIXFormsContextControl,
-                             nsIXFormsControl,
-                             nsIXFormsControlBase)
-
-
-NS_IMPL_ISUPPORTS_INHERITED3(nsXFormsBindableControlStub,
-                             nsXFormsBindableStub,
+                             nsXFormsStubElement,
                              nsIXFormsContextControl,
                              nsIXFormsControl,
                              nsIXFormsControlBase)
