@@ -45,6 +45,9 @@
 #include "xpcprivate.h"
 #include "nsString.h"
 #include "XPCNativeWrapper.h"
+#include "nsIAtom.h"
+#include "XPCWrapper.h"
+#include "nsJSPrincipals.h"
 
 //#define STRICT_CHECK_OF_UNICODE
 #ifdef STRICT_CHECK_OF_UNICODE
@@ -123,14 +126,14 @@ static intN sXPCOMUCStringFinalizerIndex = -1;
 
 // static
 JSBool
-XPCConvert::IsMethodReflectable(const nsXPTMethodInfo& info)
+XPCConvert::IsMethodReflectable(const XPTMethodDescriptor& info)
 {
-    if(info.IsNotXPCOM() || info.IsHidden())
+    if(XPT_MD_IS_NOTXPCOM(info.flags) || XPT_MD_IS_HIDDEN(info.flags))
         return JS_FALSE;
 
-    for(int i = info.GetParamCount()-1; i >= 0; i--)
+    for(int i = info.num_args-1; i >= 0; i--)
     {
-        const nsXPTParamInfo& param = info.GetParam(i);
+        const nsXPTParamInfo& param = info.params[i];
         const nsXPTType& type = param.GetType();
 
         uint8 base_type = type.TagPart();
@@ -146,15 +149,15 @@ XPCConvert::IsMethodReflectable(const nsXPTMethodInfo& info)
 /***************************************************************************/
 
 static JSBool
-GetISupportsFromJSObject(JSContext* cx, JSObject* obj, nsISupports** iface)
+GetISupportsFromJSObject(JSObject* obj, nsISupports** iface)
 {
-    JSClass* jsclass = JS_GET_CLASS(cx, obj);
+    JSClass* jsclass = STOBJ_GET_CLASS(obj);
     NS_ASSERTION(jsclass, "obj has no class");
     if(jsclass &&
        (jsclass->flags & JSCLASS_HAS_PRIVATE) &&
        (jsclass->flags & JSCLASS_PRIVATE_IS_NSISUPPORTS))
     {
-        *iface = (nsISupports*) JS_GetPrivate(cx, obj);
+        *iface = (nsISupports*) xpc_GetJSPrivate(obj);
         return JS_TRUE;
     }
     return JS_FALSE;
@@ -398,7 +401,13 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
                     if(!p)
                         return JS_FALSE;
 
-                    JSString* jsString = JS_NewUCString(cx, p, len);
+                    if(sXPCOMUCStringFinalizerIndex == -1 && 
+                       !AddXPCOMUCStringFinalizer())
+                        return JS_FALSE;
+
+                    JSString* jsString =
+                        JS_NewExternalString(cx, p, len,
+                                             sXPCOMUCStringFinalizerIndex);
 
                     if(!jsString) {
                         nsMemory::Free(p); 
@@ -462,10 +471,15 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
                     }
                     // else...
                     
+                    // XXX The OBJ_IS_NOT_GLOBAL here is not really right. In
+                    // fact, this code is depending on the fact that the
+                    // global object will not have been collected, and
+                    // therefore this NativeInterface2JSObject will not end up
+                    // creating a new XPCNativeScriptableShared.
                     nsCOMPtr<nsIXPConnectJSObjectHolder> holder;
                     if(!NativeInterface2JSObject(ccx, getter_AddRefs(holder),
                                                  iface, iid, scope, PR_TRUE,
-                                                 pErr))
+                                                 OBJ_IS_NOT_GLOBAL, pErr))
                         return JS_FALSE;
 
                     if(holder)
@@ -473,6 +487,11 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
                         JSObject* jsobj;
                         if(NS_FAILED(holder->GetJSObject(&jsobj)))
                             return JS_FALSE;
+#ifdef DEBUG
+                        if(!STOBJ_GET_PARENT(jsobj))
+                            NS_ASSERTION(STOBJ_GET_CLASS(jsobj)->flags & JSCLASS_IS_GLOBAL,
+                                         "Why did we recreate this wrapper?");
+#endif
                         *d = OBJECT_TO_JSVAL(jsobj);
                     }
                 }
@@ -651,7 +670,8 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
 
             if(!JSVAL_IS_OBJECT(s) ||
                (!(obj = JSVAL_TO_OBJECT(s))) ||
-               (!(pid = xpc_JSObjectToID(cx, obj))))
+               (!(pid = xpc_JSObjectToID(cx, obj))) ||
+               (!(pid = (const nsID*) nsMemory::Clone(pid, sizeof(nsID)))))
             {
                 return JS_FALSE;
             }
@@ -715,7 +735,7 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
                 if(str && !isNewString)
                 {
                     XPCReadableJSStringWrapper *wrapper =
-                        XPCStringConvert::JSStringToReadable(str);
+                        XPCStringConvert::JSStringToReadable(ccx, str);
                     if(!wrapper)
                         return JS_FALSE;
 
@@ -966,9 +986,28 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
                 XPCVariant* variant = XPCVariant::newVariant(ccx, s);
                 if(!variant)
                     return JS_FALSE;
-                *((nsISupports**)d) = NS_STATIC_CAST(nsIVariant*, variant);
+                *((nsISupports**)d) = static_cast<nsIVariant*>(variant);
                 return JS_TRUE;
             }
+            else if(iid->Equals(NS_GET_IID(nsIAtom)) &&
+                    JSVAL_IS_STRING(s))
+            {
+                // We're trying to pass a string as an nsIAtom.  Let's atomize!
+                JSString* str = JSVAL_TO_STRING(s);
+                PRUnichar* chars =
+                    reinterpret_cast<PRUnichar*>(JS_GetStringChars(str));
+                if (!chars) {
+                    if (pErr)
+                        *pErr = NS_ERROR_XPC_BAD_CONVERT_JS_NULL_REF;
+                    return JS_FALSE;
+                }
+                PRUint32 length = JS_GetStringLength(str);
+                nsIAtom* atom = NS_NewAtom(nsDependentString(chars, length));
+                if (!atom && pErr)
+                    *pErr = NS_ERROR_OUT_OF_MEMORY;
+                *((nsISupports**)d) = atom;
+                return atom != nsnull;                
+            }
             //else ...
 
             if(JSVAL_IS_VOID(s) || JSVAL_IS_NULL(s))
@@ -1012,6 +1051,7 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
                                      const nsID* iid,
                                      JSObject* scope,
                                      PRBool allowNativeWrapper,
+                                     PRBool isGlobal,
                                      nsresult* pErr)
 {
     NS_ASSERTION(dest, "bad param");
@@ -1038,6 +1078,8 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
     // is this a wrapped JS object?
     if(nsXPCWrappedJSClass::IsWrappedJS(src))
     {
+        NS_ASSERTION(!isGlobal, "The global object must be native");
+
         // verify that this wrapper is for the right interface
         nsCOMPtr<nsISupports> wrapper;
         if(NS_FAILED(src->QueryInterface(*iid,(void**)getter_AddRefs(wrapper))))
@@ -1061,12 +1103,17 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
 
         XPCWrappedNative* wrapper;
         nsresult rv = XPCWrappedNative::GetNewOrUsed(ccx, src, xpcscope,
-                                                     iface, &wrapper);
+                                                     iface, isGlobal,
+                                                     &wrapper);
         if(pErr)
             *pErr = rv;
         if(NS_SUCCEEDED(rv) && wrapper)
         {
-            if (allowNativeWrapper && wrapper->GetScope() != xpcscope)
+            uint32 flags = 0;
+
+            JSBool sameOrigin;
+            if (allowNativeWrapper &&
+                !xpc_SameScope(wrapper->GetScope(), xpcscope, &sameOrigin))
             {
                 // Cross scope access detected. Check if chrome code
                 // is accessing non-chrome objects, and if so, wrap
@@ -1076,49 +1123,191 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
 
                 // printf("Wrapped native accessed across scope boundary\n");
 
-                uint32 flags = JS_GetTopScriptFilenameFlags(ccx, nsnull);
-                NS_ASSERTION(flags != JSFILENAME_NULL, "null script filename");
-
-                if((flags & JSFILENAME_SYSTEM) &&
-                   !JS_IsSystemObject(ccx, wrapper->GetFlatJSObject()))
+                JSScript* script = nsnull;
+                JSObject* callee = nsnull;
+                if(ccx.GetXPCContext()->CallerTypeIsJavaScript())
                 {
-#ifdef DEBUG_XPCNativeWrapper
+                    // Called from JS.  We're going to hand the resulting
+                    // JSObject to said JS, so look for the script we want on
+                    // the stack.
+                    JSContext* cx = ccx;
+                    JSStackFrame* fp = cx->fp;
+                    while(fp)
                     {
-                        char *s = wrapper->ToString(ccx);
-                        printf("Content accessed from chrome, wrapping wrapper (%s) in XPCNativeWrapper\n", s);
-                        if (s)
-                            JS_smprintf_free(s);
+                        script = fp->script;
+                        if(script)
+                        {
+                            callee = fp->callee;
+                            break;
+                        }
+                        fp = fp->down;
                     }
+                }
+                else if(ccx.GetXPCContext()->CallerTypeIsNative())
+                {
+                    callee = ccx.GetCallee();
+                    if(callee && JS_ObjectIsFunction(ccx, callee))
+                    {
+                        // Called from c++, and calling out to |callee|, which
+                        // is a JS function object.  Look for the script for
+                        // this function.
+                        JSFunction* fun =
+                            (JSFunction*) xpc_GetJSPrivate(callee);
+                        NS_ASSERTION(fun,
+                                     "Must have JSFunction for a Function "
+                                     "object");
+                        script = JS_GetFunctionScript(ccx, fun);
+                    }
+                    else
+                    {
+                        // Else we don't know whom we're calling, so don't
+                        // create XPCNativeWrappers.
+                        callee = nsnull;
+                    }
+                }
+                // else don't create XPCNativeWrappers, since we have
+                // no idea what's calling what here.
+
+                flags = script ? JS_GetScriptFilenameFlags(script) : 0;
+                NS_ASSERTION(flags != JSFILENAME_NULL, "null script filename");
+                JSObject *flat = wrapper->GetFlatJSObject();
+
+                if(!JS_IsSystemObject(ccx, flat))
+                {
+
+                    if(flags & JSFILENAME_PROTECTED)
+                    {
+#ifdef DEBUG_XPCNativeWrapper
+                        {
+                            char *s = wrapper->ToString(ccx);
+                            printf("Content accessed from chrome, wrapping "
+                                   "wrapper (%s) in XPCNativeWrapper\n", s);
+                            if (s)
+                                JS_smprintf_free(s);
+                        }
+#endif
+                        nsIScriptSecurityManager *ssm =
+                            XPCWrapper::GetSecurityManager();
+                        nsCOMPtr<nsIPrincipal> objPrincipal;
+                        if(callee)
+                        {
+                            // Prefer getting the object princpal here.
+                            nsresult rv =
+                                ssm->GetObjectPrincipal(ccx, callee,
+                                                        getter_AddRefs(objPrincipal));
+                            if(NS_FAILED(rv))
+                                return JS_FALSE;
+                        }
+                        else
+                        {
+                            JSPrincipals *scriptPrincipal =
+                                JS_GetScriptPrincipals(ccx, script);
+                            if(scriptPrincipal)
+                            {
+                                nsJSPrincipals *nsjsp =
+                                    static_cast<nsJSPrincipals *>(scriptPrincipal);
+                                objPrincipal = nsjsp->nsIPrincipalPtr;
+                            }
+                        }
+
+                        JSObject *nativeWrapper =
+                            XPCNativeWrapper::GetNewOrUsed(ccx, wrapper,
+                                                           objPrincipal);
+
+                        if(nativeWrapper)
+                        {
+                            XPCJSObjectHolder *objHolder =
+                                XPCJSObjectHolder::newHolder(ccx, nativeWrapper);
+
+                            if (objHolder)
+                            {
+                                NS_ADDREF(objHolder);
+                                NS_RELEASE(wrapper);
+
+                                *dest = objHolder;
+                                return JS_TRUE;
+                            }
+                        }
+
+                        // Out of memory or other failure that already
+                        // threw a JS exception.
+                        NS_RELEASE(wrapper);
+                        return JS_FALSE;
+                    }
+
+                    if (flags & JSFILENAME_SYSTEM)
+                    {
+#ifdef DEBUG_mrbkap
+                        printf("Content accessed from chrome, wrapping in an "
+                               "XPCSafeJSObjectWrapper\n");
 #endif
 
-                    JSObject *nativeWrapper =
-                        XPCNativeWrapper::GetNewOrUsed(ccx, wrapper);
-
-                    if (nativeWrapper)
-                    {
-                        XPCJSObjectHolder *objHolder =
-                            XPCJSObjectHolder::newHolder(ccx, nativeWrapper);
-
-                        if (objHolder)
+                        jsval v = OBJECT_TO_JSVAL(wrapper->GetFlatJSObject());
+                        XPCJSObjectHolder *objHolder;
+                        if(!XPC_SJOW_Construct(ccx, nsnull, 1, &v, &v) ||
+                           !(objHolder = XPCJSObjectHolder::newHolder(ccx,
+                                                         JSVAL_TO_OBJECT(v))))
                         {
-                            NS_ADDREF(objHolder);
                             NS_RELEASE(wrapper);
-
-                            *dest = objHolder;
-                            return JS_TRUE;
+                            return JS_FALSE;
                         }
+
+                        NS_ADDREF(objHolder);
+                        NS_RELEASE(wrapper);
+
+                        *dest = objHolder;
+                        return JS_TRUE;
                     }
 
-                    // Out of memory
-                    NS_RELEASE(wrapper);
-                    return JS_FALSE;
+                    // Reaching across scopes from content code. Wrap
+                    // the new object in a XOW.
+                    jsval v = OBJECT_TO_JSVAL(flat);
+                    XPCJSObjectHolder *objHolder = nsnull;
+                    if(!sameOrigin)
+                    {
+                        if(!XPC_XOW_WrapObject(ccx, scope, &v) ||
+                           !(objHolder =
+                             XPCJSObjectHolder::newHolder(ccx,
+                                                          JSVAL_TO_OBJECT(v))))
+                        {
+                            NS_RELEASE(wrapper);
+                            return JS_FALSE;
+                        }
+
+                        NS_ADDREF(objHolder);
+                        NS_RELEASE(wrapper);
+                        *dest = objHolder;
+
+                        return JS_TRUE;
+                    }
                 }
             }
 
-            *dest = NS_STATIC_CAST(nsIXPConnectJSObjectHolder*, wrapper);
+            JSObject *flat = wrapper->GetFlatJSObject();
+            const char *name = STOBJ_GET_CLASS(flat)->name;
+            if(allowNativeWrapper &&
+               !(flags & JSFILENAME_SYSTEM) &&
+               !JS_IsSystemObject(ccx, flat) &&
+               XPC_XOW_ClassNeedsXOW(name))
+            {
+                jsval v = OBJECT_TO_JSVAL(flat);
+                XPCJSObjectHolder *objHolder = nsnull;
+                if (!XPC_XOW_WrapObject(ccx, scope, &v) ||
+                    !(objHolder = XPCJSObjectHolder::newHolder(ccx, JSVAL_TO_OBJECT(v))))
+                {
+                    NS_RELEASE(wrapper);
+                    return JS_FALSE;
+                }
+
+                NS_ADDREF(objHolder);
+                NS_RELEASE(wrapper);
+                *dest = objHolder;
+                return JS_TRUE;
+            }
+
+            *dest = static_cast<nsIXPConnectJSObjectHolder*>(wrapper);
             return JS_TRUE;
         }
-        
     }
     return JS_FALSE;
 }
@@ -1169,10 +1358,16 @@ XPCConvert::JSObject2NativeInterface(XPCCallContext& ccx,
         }
         // else...
         
+        // XXX E4X breaks the world. Don't try wrapping E4X objects!
+        // This hack can be removed (or changed accordingly) when the
+        // DOM <-> E4X bindings are complete, see bug 270553
+        if(JS_TypeOfValue(cx, OBJECT_TO_JSVAL(src)) == JSTYPE_XML)
+            return JS_FALSE;
+
         // Does the JSObject have 'nsISupportness'?
         // XXX hmm, I wonder if this matters anymore with no 
         // oldstyle DOM objects around.
-        if(GetISupportsFromJSObject(cx, src, &iface))
+        if(GetISupportsFromJSObject(src, &iface))
         {
             if(iface)
                 return NS_SUCCEEDED(iface->QueryInterface(*iid, dest));
@@ -1218,19 +1413,25 @@ XPCConvert::ConstructException(nsresult rv, const char* message,
     static const char format[] = "\'%s\' when calling method: [%s::%s]";
     const char * msg = message;
     char* sz = nsnull;
+    nsXPIDLString xmsg;
+    nsCAutoString sxmsg;
 
+    nsCOMPtr<nsIScriptError> errorObject = do_QueryInterface(data);
+    if(errorObject) {
+        if (NS_SUCCEEDED(errorObject->GetMessageMoz(getter_Copies(xmsg)))) {
+            CopyUTF16toUTF8(xmsg, sxmsg);
+            msg = sxmsg.get();
+        }
+    }
     if(!msg)
         if(!nsXPCException::NameAndFormatForNSResult(rv, nsnull, &msg) || ! msg)
             msg = "<error>";
-
     if(ifaceName && methodName)
-        sz = JS_smprintf(format, msg, ifaceName, methodName);
-    else
-        sz = (char*) msg; // I promise to play nice after casting away const
+        msg = sz = JS_smprintf(format, msg, ifaceName, methodName);
 
-    nsresult res = nsXPCException::NewException(sz, rv, nsnull, data, exceptn);
+    nsresult res = nsXPCException::NewException(msg, rv, nsnull, data, exceptn);
 
-    if(sz && sz != msg)
+    if(sz)
         JS_smprintf_free(sz);
     return res;
 }
@@ -1319,8 +1520,8 @@ XPCConvert::JSValToXPCException(XPCCallContext& ccx,
                                                 nsnull, &jswrapper);
                     if(NS_FAILED(rv))
                         return rv;
-                    *exceptn = NS_REINTERPRET_CAST(nsIException*,
-                                                   jswrapper);
+                    *exceptn = reinterpret_cast<nsIException*>
+                                               (jswrapper);
                     return NS_OK;
                 }
             }
@@ -1368,17 +1569,14 @@ XPCConvert::JSValToXPCException(XPCCallContext& ccx,
         }
         else
         {
-            if(JSVAL_TO_DOUBLE(s))
+            number = *(JSVAL_TO_DOUBLE(s));
+            if(number > 0.0 &&
+               number < (double)0xffffffff &&
+               0.0 == fmod(number,1))
             {
-                number = *(JSVAL_TO_DOUBLE(s));
-                if(number > 0.0 &&
-                   number < (double)0xffffffff &&
-                   0.0 == fmod(number,1))
-                {
-                    rv = (nsresult) number;
-                    if(NS_FAILED(rv))
-                        isResult = JS_TRUE;
-                }
+                rv = (nsresult) number;
+                if(NS_FAILED(rv))
+                    isResult = JS_TRUE;
             }
         }
 
@@ -1450,7 +1648,7 @@ XPCConvert::JSErrorToXPCException(XPCCallContext& ccx,
 
         NS_ADDREF(data);
         data->Init(bestMessage.get(),
-                   NS_ConvertASCIItoUCS2(report->filename).get(),
+                   NS_ConvertASCIItoUTF16(report->filename).get(),
                    (const PRUnichar *)report->uclinebuf, report->lineno,
                    report->uctokenptr - report->uclinebuf, report->flags,
                    "XPConnect JavaScript");
@@ -1617,7 +1815,8 @@ XPCConvert::NativeArray2JS(XPCCallContext& ccx,
         *pErr = NS_ERROR_XPC_BAD_CONVERT_NATIVE;
 
     JSUint32 i;
-    jsval current;
+    jsval current = JSVAL_NULL;
+    AUTO_MARK_JSVAL(ccx, &current);
 
 #define POPULATE(_t)                                                         \
     PR_BEGIN_MACRO                                                           \
