@@ -42,6 +42,9 @@
 /* Implement global service to track stack of JSContext per thread. */
 
 #include "xpcprivate.h"
+#include "XPCWrapper.h"
+#include "nsDOMJSUtils.h"
+#include "nsIScriptGlobalObject.h"
 
 /***************************************************************************/
 
@@ -106,17 +109,36 @@ XPCJSContextStack::Pop(JSContext * *_retval)
     if(idx > 0)
     {
         --idx; // Advance to new top of the stack
-        JSContextAndFrame & e = mStack[idx];
+        XPCJSContextInfo & e = mStack[idx];
         NS_ASSERTION(!e.frame || e.cx, "Shouldn't have frame without a cx!");
-        if(e.cx)
+        if(e.cx && e.frame)
         {
             JS_RestoreFrameChain(e.cx, e.frame);
             e.frame = nsnull;
         }
+
+        if(e.requestDepth)
+            JS_ResumeRequest(e.cx, e.requestDepth);
+
+        e.requestDepth = 0;
     }
     return NS_OK;
 }
 
+static nsIPrincipal*
+GetPrincipalFromCx(JSContext *cx)
+{
+    nsIScriptContext* scriptContext = GetScriptContextFromJSContext(cx);
+    if (scriptContext)
+    {
+        nsCOMPtr<nsIScriptObjectPrincipal> globalData =
+            do_QueryInterface(scriptContext->GetGlobalObject());
+        if (globalData)
+            return globalData->GetPrincipal();
+    }
+    return nsnull;
+}
+
 /* void push (in JSContext cx); */
 NS_IMETHODIMP
 XPCJSContextStack::Push(JSContext * cx)
@@ -125,9 +147,34 @@ XPCJSContextStack::Push(JSContext * cx)
         return NS_ERROR_OUT_OF_MEMORY;
     if(mStack.Length() > 1)
     {
-        JSContextAndFrame & e = mStack[mStack.Length() - 2];
+        XPCJSContextInfo & e = mStack[mStack.Length() - 2];
         if(e.cx)
+        {
+            if (e.cx == cx)
+            {
+                nsIScriptSecurityManager* ssm = XPCWrapper::GetSecurityManager();
+                if (ssm)
+                {
+                    nsIPrincipal* globalObjectPrincipal =
+                        GetPrincipalFromCx(cx);
+                    if (globalObjectPrincipal)
+                    {
+                        nsIPrincipal* subjectPrincipal = ssm->GetCxSubjectPrincipal(cx);
+                        PRBool equals = PR_FALSE;
+                        globalObjectPrincipal->Equals(subjectPrincipal, &equals);
+                        if (equals)
+                        {
+                            return NS_OK;
+                        }
+                    }
+                }
+            }
+
             e.frame = JS_SaveFrameChain(e.cx);
+
+            if(e.cx != cx && JS_GetContextThread(e.cx))
+                e.requestDepth = JS_SuspendRequest(e.cx);
+        }
     }
     return NS_OK;
 }
@@ -150,10 +197,25 @@ SafeGlobalResolve(JSContext *cx, JSObject *obj, jsval id)
     return JS_ResolveStandardClass(cx, obj, id, &resolved);
 }
 
+JS_STATIC_DLL_CALLBACK(void)
+SafeFinalize(JSContext* cx, JSObject* obj)
+{
+#ifndef XPCONNECT_STANDALONE
+    nsIScriptObjectPrincipal* sop =
+        static_cast<nsIScriptObjectPrincipal*>(xpc_GetJSPrivate(obj));
+    NS_IF_RELEASE(sop);
+#endif
+}
+
 static JSClass global_class = {
-    "global_for_XPCJSContextStack_SafeJSContext", 0,
+    "global_for_XPCJSContextStack_SafeJSContext",
+#ifndef XPCONNECT_STANDALONE
+    JSCLASS_HAS_PRIVATE | JSCLASS_PRIVATE_IS_NSISUPPORTS | JSCLASS_GLOBAL_FLAGS,
+#else
+    0,
+#endif
     JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
-    JS_EnumerateStub, SafeGlobalResolve, JS_ConvertStub, JS_FinalizeStub,
+    JS_EnumerateStub, SafeGlobalResolve, JS_ConvertStub, SafeFinalize,
     JSCLASS_NO_OPTIONAL_MEMBERS
 };
 
@@ -163,11 +225,28 @@ XPCJSContextStack::GetSafeJSContext(JSContext * *aSafeJSContext)
 {
     if(!mSafeJSContext)
     {
+#ifndef XPCONNECT_STANDALONE
+        // Start by getting the principal holder and principal for this
+        // context.  If we can't manage that, don't bother with the rest.
+        nsCOMPtr<nsIPrincipal> principal =
+            do_CreateInstance("@mozilla.org/nullprincipal;1");
+        nsCOMPtr<nsIScriptObjectPrincipal> sop;
+        if(principal)
+        {
+            sop = new PrincipalHolder(principal);
+        }
+        if(!sop)
+        {
+            *aSafeJSContext = nsnull;
+            return NS_ERROR_FAILURE;
+        }        
+#endif /* !XPCONNECT_STANDALONE */
+        
         JSRuntime *rt;
         XPCJSRuntime* xpcrt;
 
         nsXPConnect* xpc = nsXPConnect::GetXPConnect();
-        nsCOMPtr<nsIXPConnect> xpcholder(NS_STATIC_CAST(nsIXPConnect*, xpc));
+        nsCOMPtr<nsIXPConnect> xpcholder(static_cast<nsIXPConnect*>(xpc));
 
         if(xpc && (xpcrt = xpc->GetRuntime()) && (rt = xpcrt->GetJSRuntime()))
         {
@@ -178,6 +257,27 @@ XPCJSContextStack::GetSafeJSContext(JSContext * *aSafeJSContext)
                 AutoJSRequestWithNoCallContext req(mSafeJSContext);
                 JSObject *glob;
                 glob = JS_NewObject(mSafeJSContext, &global_class, NULL, NULL);
+
+#ifndef XPCONNECT_STANDALONE
+                if(glob)
+                {
+                    // Note: make sure to set the private before calling
+                    // InitClasses
+                    nsIScriptObjectPrincipal* priv = nsnull;
+                    sop.swap(priv);
+                    if(!JS_SetPrivate(mSafeJSContext, glob, priv))
+                    {
+                        // Drop the whole thing
+                        NS_RELEASE(priv);
+                        glob = nsnull;
+                    }
+                }
+
+                // After this point either glob is null and the
+                // nsIScriptObjectPrincipal ownership is either handled by the
+                // nsCOMPtr or dealt with, or we'll release in the finalize
+                // hook.
+#endif
                 if(!glob || NS_FAILED(xpc->InitClasses(mSafeJSContext, glob)))
                 {
                     // Explicitly end the request since we are about to kill
@@ -331,7 +431,7 @@ nsXPCThreadJSContextStackImpl::Pop(JSContext * *_retval)
 NS_IMETHODIMP
 nsXPCThreadJSContextStackImpl::Push(JSContext * cx)
 {
-    XPCJSContextStack* myStack = GetStackForCurrentThread();
+    XPCJSContextStack* myStack = GetStackForCurrentThread(cx);
 
     if(!myStack)
         return NS_ERROR_FAILURE;
@@ -360,7 +460,7 @@ nsXPCThreadJSContextStackImpl::GetSafeJSContext(JSContext * *aSafeJSContext)
 NS_IMETHODIMP
 nsXPCThreadJSContextStackImpl::SetSafeJSContext(JSContext * aSafeJSContext)
 {
-    XPCJSContextStack* myStack = GetStackForCurrentThread();
+    XPCJSContextStack* myStack = GetStackForCurrentThread(aSafeJSContext);
 
     if(!myStack)
         return NS_ERROR_FAILURE;
@@ -370,9 +470,11 @@ nsXPCThreadJSContextStackImpl::SetSafeJSContext(JSContext * aSafeJSContext)
 
 /***************************************************************************/
 
-PRUintn           XPCPerThreadData::gTLSIndex = BAD_TLS_INDEX;
-PRLock*           XPCPerThreadData::gLock     = nsnull;
-XPCPerThreadData* XPCPerThreadData::gThreads  = nsnull;
+PRUintn           XPCPerThreadData::gTLSIndex       = BAD_TLS_INDEX;
+PRLock*           XPCPerThreadData::gLock           = nsnull;
+XPCPerThreadData* XPCPerThreadData::gThreads        = nsnull;
+XPCPerThreadData *XPCPerThreadData::sMainThreadData = nsnull;
+void *            XPCPerThreadData::sMainJSThread   = nsnull;
 
 static jsuword
 GetThreadStackLimit()
@@ -395,6 +497,8 @@ GetThreadStackLimit()
   return stackLimit;
 }
 
+MOZ_DECL_CTOR_COUNTER(xpcPerThreadData)
+
 XPCPerThreadData::XPCPerThreadData()
     :   mJSContextStack(new XPCJSContextStack()),
         mNextThread(nsnull),
@@ -412,6 +516,7 @@ XPCPerThreadData::XPCPerThreadData()
       , mWrappedNativeThreadsafetyReportDepth(0)
 #endif
 {
+    MOZ_COUNT_CTOR(xpcPerThreadData);
     if(gLock)
     {
         nsAutoLock lock(gLock);
@@ -436,6 +541,8 @@ XPCPerThreadData::Cleanup()
 
 XPCPerThreadData::~XPCPerThreadData()
 {
+    MOZ_COUNT_DTOR(xpcPerThreadData);
+
     Cleanup();
 
     // Unlink 'this' from the list of threads.
@@ -474,7 +581,7 @@ xpc_ThreadDataDtorCB(void* ptr)
         delete data;
 }
 
-void XPCPerThreadData::MarkAutoRootsBeforeJSFinalize(JSContext* cx)
+void XPCPerThreadData::TraceJS(JSTracer *trc)
 {
 #ifdef XPC_TRACK_AUTOMARKINGPTR_STATS
     {
@@ -490,7 +597,7 @@ void XPCPerThreadData::MarkAutoRootsBeforeJSFinalize(JSContext* cx)
 #endif
 
     if(mAutoRoots)
-        mAutoRoots->MarkBeforeJSFinalize(cx);
+        mAutoRoots->TraceJS(trc);
 }
 
 void XPCPerThreadData::MarkAutoRootsAfterJSFinalize()
@@ -501,7 +608,7 @@ void XPCPerThreadData::MarkAutoRootsAfterJSFinalize()
 
 // static
 XPCPerThreadData*
-XPCPerThreadData::GetData()
+XPCPerThreadData::GetDataImpl(JSContext *cx)
 {
     XPCPerThreadData* data;
 
@@ -546,6 +653,14 @@ XPCPerThreadData::GetData()
             return nsnull;
         }
     }
+
+    if(cx && !sMainJSThread && NS_IsMainThread())
+    {
+        sMainJSThread = cx->thread;
+
+        sMainThreadData = data;
+    }
+
     return data;
 }
 
@@ -609,7 +724,7 @@ nsXPCJSContextStackIterator::Reset(nsIJSContextStack *aStack)
 {
     // XXX This is pretty ugly.
     nsXPCThreadJSContextStackImpl *impl =
-        NS_STATIC_CAST(nsXPCThreadJSContextStackImpl*, aStack);
+        static_cast<nsXPCThreadJSContextStackImpl*>(aStack);
     XPCJSContextStack *stack = impl->GetStackForCurrentThread();
     if(!stack)
         return NS_ERROR_FAILURE;
