@@ -35,7 +35,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: sessobj.c,v $ $Revision: 1.13 $ $Date: 2007/01/05 00:23:14 $";
+static const char CVS_ID[] = "@(#) $RCSfile: sessobj.c,v $ $Revision: 1.13.46.1 $ $Date: 2009/07/31 02:16:06 $";
 #endif /* DEBUG */
 
 /*
@@ -271,14 +271,14 @@ nssCKMDSessionObject_Create
   *pError = CKR_OK;
 
   mdso = nss_ZNEW(arena, nssCKMDSessionObject);
-  if( (nssCKMDSessionObject *)NULL == mdso ) {
+  if (!mdso) {
     goto loser;
   }
 
   mdso->arena = arena;
   mdso->n = ulCount;
   mdso->attributes = nss_ZNEWARRAY(arena, NSSItem, ulCount);
-  if( (NSSItem *)NULL == mdso->attributes ) {
+  if (!mdso->attributes) {
     goto loser;
   }
 
@@ -290,7 +290,7 @@ nssCKMDSessionObject_Create
     mdso->types[i] = attributes[i].type;
     mdso->attributes[i].size = attributes[i].ulValueLen;
     mdso->attributes[i].data = nss_ZAlloc(arena, attributes[i].ulValueLen);
-    if( (void *)NULL == mdso->attributes[i].data ) {
+    if (!mdso->attributes[i].data) {
       goto loser;
     }
     (void)nsslibc_memcpy(mdso->attributes[i].data, attributes[i].pValue,
@@ -298,7 +298,7 @@ nssCKMDSessionObject_Create
   }
 
   mdObject = nss_ZNEW(arena, NSSCKMDObject);
-  if( (NSSCKMDObject *)NULL == mdObject ) {
+  if (!mdObject) {
     goto loser;
   }
 
@@ -314,7 +314,7 @@ nssCKMDSessionObject_Create
   mdObject->GetObjectSize = nss_ckmdSessionObject_GetObjectSize;
 
   hash = nssCKFWToken_GetSessionObjectHash(fwToken);
-  if( (nssCKFWHash *)NULL == hash ) {
+  if (!hash) {
     *pError = CKR_GENERAL_ERROR;
     goto loser;
   }
@@ -335,8 +335,8 @@ nssCKMDSessionObject_Create
   return mdObject;
 
  loser:
-  if( (nssCKMDSessionObject *)NULL != mdso ) {
-    if( (NSSItem *)NULL != mdso->attributes ) {
+  if (mdso) {
+    if (mdso->attributes) {
       for( i = 0; i < ulCount; i++ ) {
         nss_ZFreeIf(mdso->attributes[i].data);
       }
@@ -475,7 +475,7 @@ nss_ckmdSessionObject_GetAttributeCount
   nssCKMDSessionObject *obj;
 
 #ifdef NSSDEBUG
-  if( (CK_RV *)NULL == pError ) {
+  if (!pError) {
     return 0;
   }
 
@@ -560,7 +560,7 @@ nss_ckmdSessionObject_GetAttributeSize
   CK_ULONG i;
 
 #ifdef NSSDEBUG
-  if( (CK_RV *)NULL == pError ) {
+  if (!pError) {
     return 0;
   }
 
@@ -610,7 +610,7 @@ nss_ckmdSessionObject_GetAttribute
   item.needsFreeing = PR_FALSE;
   item.item = NULL;
 #ifdef NSSDEBUG
-  if( (CK_RV *)NULL == pError ) {
+  if (!pError) {
     return item;
   }
 
@@ -684,7 +684,7 @@ nss_ckmdSessionObject_SetAttribute
 
   n.size = value->size;
   n.data = nss_ZAlloc(obj->arena, n.size);
-  if( (void *)NULL == n.data ) {
+  if (!n.data) {
     return CKR_HOST_MEMORY;
   }
   (void)nsslibc_memcpy(n.data, value->data, n.size);
@@ -702,7 +702,7 @@ nss_ckmdSessionObject_SetAttribute
    */
 
   ra = (NSSItem *)nss_ZRealloc(obj->attributes, sizeof(NSSItem) * (obj->n + 1));
-  if( (NSSItem *)NULL == ra ) {
+  if (!ra) {
     nss_ZFreeIf(n.data);
     return CKR_HOST_MEMORY;
   }
@@ -711,7 +711,7 @@ nss_ckmdSessionObject_SetAttribute
   if( (CK_ATTRIBUTE_TYPE_PTR)NULL == rt ) {
     nss_ZFreeIf(n.data);
     obj->attributes = (NSSItem *)nss_ZRealloc(ra, sizeof(NSSItem) * obj->n);
-    if( (NSSItem *)NULL == obj->attributes ) {
+    if (!obj->attributes) {
       return CKR_GENERAL_ERROR;
     }
     return CKR_HOST_MEMORY;
@@ -749,7 +749,7 @@ nss_ckmdSessionObject_GetObjectSize
   CK_ULONG rv = (CK_ULONG)0;
 
 #ifdef NSSDEBUG
-  if( (CK_RV *)NULL == pError ) {
+  if (!pError) {
     return 0;
   }
 
@@ -967,7 +967,7 @@ nssCKMDFindSessionObjects_Create
   NSSCKMDFindObjects *rv;
 
 #ifdef NSSDEBUG
-  if( (CK_RV *)NULL == pError ) {
+  if (!pError) {
     return (NSSCKMDFindObjects *)NULL;
   }
 
@@ -985,19 +985,19 @@ nssCKMDFindSessionObjects_Create
   *pError = CKR_OK;
 
   hash = nssCKFWToken_GetSessionObjectHash(fwToken);
-  if( (nssCKFWHash *)NULL == hash ) {
+  if (!hash) {
     *pError= CKR_GENERAL_ERROR;
     return (NSSCKMDFindObjects *)NULL;
   }
 
   arena = NSSArena_Create();
-  if( (NSSArena *)NULL == arena ) {
+  if (!arena) {
     *pError = CKR_HOST_MEMORY;
     return (NSSCKMDFindObjects *)NULL;
   }
 
   mdfso = nss_ZNEW(arena, nssCKMDFindSessionObjects);
-  if( (nssCKMDFindSessionObjects *)NULL == mdfso ) {
+  if (!mdfso) {
     goto loser;
   }
 
@@ -1097,7 +1097,7 @@ nss_ckmdFindSessionObjects_Next
 
   mdfso = (nssCKMDFindSessionObjects *)mdFindObjects->etc;
 
-  while( (NSSCKMDObject *)NULL == rv ) {
+  while (!rv) {
     if( (struct nodeStr *)NULL == mdfso->list ) {
       *pError = CKR_OK;
       return (NSSCKMDObject *)NULL;
