@@ -1,38 +1,41 @@
-/* 
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- * 
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
- * 
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
  * The Original Code is the Netscape security libraries.
- * 
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are 
- * Copyright (C) 1994-2000 Netscape Communications Corporation.  All
- * Rights Reserved.
- * 
+ *
+ * The Initial Developer of the Original Code is
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1994-2000
+ * the Initial Developer. All Rights Reserved.
+ *
  * Contributor(s):
- * 
- * Alternatively, the contents of this file may be used under the
- * terms of the GNU General Public License Version 2 or later (the
- * "GPL"), in which case the provisions of the GPL are applicable 
- * instead of those above.  If you wish to allow use of your 
- * version of this file only under the terms of the GPL and not to
- * allow others to use your version of this file under the MPL,
- * indicate your decision by deleting the provisions above and
- * replace them with the notice and other provisions required by
- * the GPL.  If you do not delete the provisions above, a recipient
- * may use your version of this file under either the MPL or the
- * GPL.
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: slot.c,v $ $Revision: 1.4 $ $Date: 2002/10/10 22:39:23 $ $Name: FIREFOX_1_0_7_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: slot.c,v $ $Revision: 1.6.172.1 $ $Date: 2009/07/31 02:16:06 $";
 #endif /* DEBUG */
 
 /*
@@ -169,7 +172,7 @@ nssCKFWSlot_Create
   NSSArena *arena;
 
 #ifdef NSSDEBUG
-  if( (CK_RV *)NULL == pError ) {
+  if (!pError) {
     return (NSSCKFWSlot *)NULL;
   }
 
@@ -180,20 +183,20 @@ nssCKFWSlot_Create
 #endif /* NSSDEBUG */
 
   mdInstance = nssCKFWInstance_GetMDInstance(fwInstance);
-  if( (NSSCKMDInstance *)NULL == mdInstance ) {
+  if (!mdInstance) {
     *pError = CKR_GENERAL_ERROR;
     return (NSSCKFWSlot *)NULL;
   }
 
   arena = nssCKFWInstance_GetArena(fwInstance, pError);
-  if( (NSSArena *)NULL == arena ) {
+  if (!arena) {
     if( CKR_OK == *pError ) {
       *pError = CKR_GENERAL_ERROR;
     }
   }
 
   fwSlot = nss_ZNEW(arena, NSSCKFWSlot);
-  if( (NSSCKFWSlot *)NULL == fwSlot ) {
+  if (!fwSlot) {
     *pError = CKR_HOST_MEMORY;
     return (NSSCKFWSlot *)NULL;
   }
@@ -204,7 +207,7 @@ nssCKFWSlot_Create
   fwSlot->slotID = slotID;
 
   fwSlot->mutex = nssCKFWInstance_CreateMutex(fwInstance, arena, pError);
-  if( (NSSCKFWMutex *)NULL == fwSlot->mutex ) {
+  if (!fwSlot->mutex) {
     if( CKR_OK == *pError ) {
       *pError = CKR_GENERAL_ERROR;
     }
@@ -212,7 +215,7 @@ nssCKFWSlot_Create
     return (NSSCKFWSlot *)NULL;
   }
 
-  if( (void *)NULL != (void *)mdSlot->Initialize ) {
+  if (mdSlot->Initialize) {
     *pError = CKR_OK;
     *pError = mdSlot->Initialize(mdSlot, fwSlot, mdInstance, fwInstance);
     if( CKR_OK != *pError ) {
@@ -225,7 +228,7 @@ nssCKFWSlot_Create
 #ifdef DEBUG
   *pError = slot_add_pointer(fwSlot);
   if( CKR_OK != *pError ) {
-    if( (void *)NULL != (void *)mdSlot->Destroy ) {
+    if (mdSlot->Destroy) {
       mdSlot->Destroy(mdSlot, fwSlot, mdInstance, fwInstance);
     }
 
@@ -262,7 +265,7 @@ nssCKFWSlot_Destroy
 
   (void)nssCKFWMutex_Destroy(fwSlot->mutex);
 
-  if( (void *)NULL != (void *)fwSlot->mdSlot->Destroy ) {
+  if (fwSlot->mdSlot->Destroy) {
     fwSlot->mdSlot->Destroy(fwSlot->mdSlot, fwSlot, 
       fwSlot->mdInstance, fwSlot->fwInstance);
   }
@@ -381,12 +384,12 @@ nssCKFWSlot_GetSlotDescription
     return error;
   }
 
-  if( (NSSUTF8 *)NULL == fwSlot->slotDescription ) {
-    if( (void *)NULL != (void *)fwSlot->mdSlot->GetSlotDescription ) {
+  if (!fwSlot->slotDescription) {
+    if (fwSlot->mdSlot->GetSlotDescription) {
       fwSlot->slotDescription = fwSlot->mdSlot->GetSlotDescription(
         fwSlot->mdSlot, fwSlot, fwSlot->mdInstance, 
         fwSlot->fwInstance, &error);
-      if( ((NSSUTF8 *)NULL == fwSlot->slotDescription) && (CKR_OK != error) ) {
+      if ((!fwSlot->slotDescription) && (CKR_OK != error)) {
         goto done;
       }
     } else {
@@ -431,12 +434,12 @@ nssCKFWSlot_GetManufacturerID
     return error;
   }
 
-  if( (NSSUTF8 *)NULL == fwSlot->manufacturerID ) {
-    if( (void *)NULL != (void *)fwSlot->mdSlot->GetManufacturerID ) {
+  if (!fwSlot->manufacturerID) {
+    if (fwSlot->mdSlot->GetManufacturerID) {
       fwSlot->manufacturerID = fwSlot->mdSlot->GetManufacturerID(
         fwSlot->mdSlot, fwSlot, fwSlot->mdInstance, 
         fwSlot->fwInstance, &error);
-      if( ((NSSUTF8 *)NULL == fwSlot->manufacturerID) && (CKR_OK != error) ) {
+      if ((!fwSlot->manufacturerID) && (CKR_OK != error)) {
         goto done;
       }
     } else {
@@ -468,7 +471,7 @@ nssCKFWSlot_GetTokenPresent
   }
 #endif /* NSSDEBUG */
 
-  if( (void *)NULL == (void *)fwSlot->mdSlot->GetTokenPresent ) {
+  if (!fwSlot->mdSlot->GetTokenPresent) {
     return CK_TRUE;
   }
 
@@ -492,7 +495,7 @@ nssCKFWSlot_GetRemovableDevice
   }
 #endif /* NSSDEBUG */
 
-  if( (void *)NULL == (void *)fwSlot->mdSlot->GetRemovableDevice ) {
+  if (!fwSlot->mdSlot->GetRemovableDevice) {
     return CK_FALSE;
   }
 
@@ -516,7 +519,7 @@ nssCKFWSlot_GetHardwareSlot
   }
 #endif /* NSSDEBUG */
 
-  if( (void *)NULL == (void *)fwSlot->mdSlot->GetHardwareSlot ) {
+  if (!fwSlot->mdSlot->GetHardwareSlot) {
     return CK_FALSE;
   }
 
@@ -554,7 +557,7 @@ nssCKFWSlot_GetHardwareVersion
     goto done;
   }
 
-  if( (void *)NULL != (void *)fwSlot->mdSlot->GetHardwareVersion ) {
+  if (fwSlot->mdSlot->GetHardwareVersion) {
     fwSlot->hardwareVersion = fwSlot->mdSlot->GetHardwareVersion(
       fwSlot->mdSlot, fwSlot, fwSlot->mdInstance, fwSlot->fwInstance);
   } else {
@@ -598,7 +601,7 @@ nssCKFWSlot_GetFirmwareVersion
     goto done;
   }
 
-  if( (void *)NULL != (void *)fwSlot->mdSlot->GetFirmwareVersion ) {
+  if (fwSlot->mdSlot->GetFirmwareVersion) {
     fwSlot->firmwareVersion = fwSlot->mdSlot->GetFirmwareVersion(
       fwSlot->mdSlot, fwSlot, fwSlot->mdInstance, fwSlot->fwInstance);
   } else {
@@ -627,7 +630,7 @@ nssCKFWSlot_GetToken
   NSSCKFWToken *fwToken;
 
 #ifdef NSSDEBUG
-  if( (CK_RV *)NULL == pError ) {
+  if (!pError) {
     return (NSSCKFWToken *)NULL;
   }
 
@@ -642,8 +645,8 @@ nssCKFWSlot_GetToken
     return (NSSCKFWToken *)NULL;
   }
 
-  if( (NSSCKFWToken *)NULL == fwSlot->fwToken ) {
-    if( (void *)NULL == (void *)fwSlot->mdSlot->GetToken ) {
+  if (!fwSlot->fwToken) {
+    if (!fwSlot->mdSlot->GetToken) {
       *pError = CKR_GENERAL_ERROR;
       fwToken = (NSSCKFWToken *)NULL;
       goto done;
@@ -651,7 +654,7 @@ nssCKFWSlot_GetToken
 
     mdToken = fwSlot->mdSlot->GetToken(fwSlot->mdSlot, fwSlot,
       fwSlot->mdInstance, fwSlot->fwInstance, pError);
-    if( (NSSCKMDToken *)NULL == mdToken ) {
+    if (!mdToken) {
       if( CKR_OK == *pError ) {
         *pError = CKR_GENERAL_ERROR;
       }
