@@ -1,38 +1,41 @@
-/* 
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- * 
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
- * 
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
  * The Original Code is the Netscape security libraries.
- * 
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are 
- * Copyright (C) 1994-2000 Netscape Communications Corporation.  All
- * Rights Reserved.
- * 
+ *
+ * The Initial Developer of the Original Code is
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1994-2000
+ * the Initial Developer. All Rights Reserved.
+ *
  * Contributor(s):
- * 
- * Alternatively, the contents of this file may be used under the
- * terms of the GNU General Public License Version 2 or later (the
- * "GPL"), in which case the provisions of the GPL are applicable 
- * instead of those above.  If you wish to allow use of your 
- * version of this file only under the terms of the GPL and not to
- * allow others to use your version of this file under the MPL,
- * indicate your decision by deleting the provisions above and
- * replace them with the notice and other provisions required by
- * the GPL.  If you do not delete the provisions above, a recipient
- * may use your version of this file under either the MPL or the
- * GPL.
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.49 $ $Date: 2003/11/15 00:09:51 $ $Name: FIREFOX_1_0_6_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.65.20.1 $ $Date: 2009/07/31 02:16:47 $";
 #endif /* DEBUG */
 
 #ifndef NSSPKI_H
@@ -53,11 +56,9 @@ static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.49 $
 
 #include "pkistore.h"
 
-#ifdef NSS_3_4_CODE
 #include "pki3hack.h"
 #include "pk11func.h"
 #include "hasht.h"
-#endif
 
 #ifndef BASE_H
 #include "base.h"
@@ -73,9 +74,11 @@ nssCertificate_Create (
 {
     PRStatus status;
     NSSCertificate *rvCert;
-    /* mark? */
+    nssArenaMark * mark;
     NSSArena *arena = object->arena;
     PR_ASSERT(object->instances != NULL && object->numInstances > 0);
+    PR_ASSERT(object->lockType == nssPKIMonitor);
+    mark = nssArena_Mark(arena);
     rvCert = nss_ZNEW(arena, NSSCertificate);
     if (!rvCert) {
 	return (NSSCertificate *)NULL;
@@ -91,13 +94,19 @@ nssCertificate_Create (
                                                   &rvCert->issuer,
                                                   &rvCert->serial,
                                                   &rvCert->subject);
-    if (status != PR_SUCCESS) {
-	return (NSSCertificate *)NULL;
-    }
-    /* all certs need an encoding value */
-    if (rvCert->encoding.data == NULL) {
+    if (status != PR_SUCCESS ||
+	!rvCert->encoding.data ||
+	!rvCert->encoding.size ||
+	!rvCert->issuer.data ||
+	!rvCert->issuer.size ||
+	!rvCert->serial.data ||
+	!rvCert->serial.size) {
+	if (mark)
+	    nssArena_Release(arena, mark);
 	return (NSSCertificate *)NULL;
     }
+    if (mark)
+	nssArena_Unmark(arena, mark);
     return rvCert;
 }
 
@@ -117,6 +126,9 @@ nssCertificate_Destroy (
   NSSCertificate *c
 )
 {
+    nssCertificateStoreTrace lockTrace = {NULL, NULL, PR_FALSE, PR_FALSE};
+    nssCertificateStoreTrace unlockTrace = {NULL, NULL, PR_FALSE, PR_FALSE};
+
     if (c) {
 	PRUint32 i;
 	nssDecodedCert *dc = c->decoding;
@@ -127,7 +139,7 @@ nssCertificate_Destroy (
 
 	/* --- LOCK storage --- */
 	if (cc) {
-	    nssCertificateStore_Lock(cc->certStore);
+	    nssCertificateStore_Lock(cc->certStore, &lockTrace);
 	} else {
 	    nssTrustDomain_LockCertCache(td);
 	}
@@ -135,7 +147,8 @@ nssCertificate_Destroy (
 	    /* --- remove cert and UNLOCK storage --- */
 	    if (cc) {
 		nssCertificateStore_RemoveCertLOCKED(cc->certStore, c);
-		nssCertificateStore_Unlock(cc->certStore);
+		nssCertificateStore_Unlock(cc->certStore, &lockTrace,
+                                           &unlockTrace);
 	    } else {
 		nssTrustDomain_RemoveCertFromCacheLOCKED(td, c);
 		nssTrustDomain_UnlockCertCache(td);
@@ -144,13 +157,15 @@ nssCertificate_Destroy (
 	    for (i=0; i<c->object.numInstances; i++) {
 		nssCryptokiObject_Destroy(c->object.instances[i]);
 	    }
-	    PZ_DestroyLock(c->object.lock);
+	    nssPKIObject_DestroyLock(&c->object);
 	    nssArena_Destroy(c->object.arena);
 	    nssDecodedCert_Destroy(dc);
 	} else {
 	    /* --- UNLOCK storage --- */
 	    if (cc) {
-		nssCertificateStore_Unlock(cc->certStore);
+		nssCertificateStore_Unlock(cc->certStore,
+					   &lockTrace,
+					   &unlockTrace);
 	    } else {
 		nssTrustDomain_UnlockCertCache(td);
 	    }
@@ -301,25 +316,20 @@ nssCertificate_GetDecoding (
   NSSCertificate *c
 )
 {
-    /* There is a race in assigning c->decoding.  
-    ** This is a workaround.  Bugzilla bug 225525.
-    */
-    if (!c->decoding) {
-	nssDecodedCert * deco =
-	    nssDecodedCert_Create(NULL, &c->encoding, c->type);
-	/* Once this race is fixed, an assertion should be put 
-	** here to detect any regressions. 
-    	PORT_Assert(!c->decoding); 
-	*/
-	if (!c->decoding) {
-	    /* we won the race. Use our copy. */
-	    c->decoding = deco;
-        } else {
-	    /* we lost the race.  discard deco. */
-	    nssDecodedCert_Destroy(deco);
-	}
+    nssDecodedCert* deco = NULL;
+    if (c->type == NSSCertificateType_PKIX) {
+        (void)STAN_GetCERTCertificate(c);
     }
-    return c->decoding;
+    nssPKIObject_Lock(&c->object);
+    if (!c->decoding) {
+	deco = nssDecodedCert_Create(NULL, &c->encoding, c->type);
+    	PORT_Assert(!c->decoding); 
+        c->decoding = deco;
+    } else {
+        deco = c->decoding;
+    }
+    nssPKIObject_Unlock(&c->object);
+    return deco;
 }
 
 static NSSCertificate **
@@ -401,7 +411,9 @@ find_cert_issuer (
   NSSCertificate *c,
   NSSTime *timeOpt,
   NSSUsage *usage,
-  NSSPolicies *policiesOpt
+  NSSPolicies *policiesOpt,
+  NSSTrustDomain *td,
+  NSSCryptoContext *cc
 )
 {
     NSSArena *arena;
@@ -409,15 +421,11 @@ find_cert_issuer (
     NSSCertificate **ccIssuers = NULL;
     NSSCertificate **tdIssuers = NULL;
     NSSCertificate *issuer = NULL;
-    NSSTrustDomain *td;
-    NSSCryptoContext *cc;
-    cc = c->object.cryptoContext; /* NSSCertificate_GetCryptoContext(c); */
-    td = NSSCertificate_GetTrustDomain(c);
-#ifdef NSS_3_4_CODE
-    if (!td) {
-	td = STAN_GetDefaultTrustDomain();
-    }
-#endif
+
+    if (!cc)
+	cc = c->object.cryptoContext;
+    if (!td)
+	td = NSSCertificate_GetTrustDomain(c);
     arena = nssArena_Create();
     if (!arena) {
 	return (NSSCertificate *)NULL;
@@ -429,7 +437,8 @@ find_cert_issuer (
 	                                                       0,
 	                                                       arena);
     }
-    tdIssuers = nssTrustDomain_FindCertificatesBySubject(td,
+    if (td)
+	tdIssuers = nssTrustDomain_FindCertificatesBySubject(td,
                                                          &c->issuer,
                                                          NULL,
                                                          0,
@@ -442,6 +451,10 @@ find_cert_issuer (
 	if (dc) {
 	    issuerID = dc->getIssuerIdentifier(dc);
 	}
+	/* XXX review based on CERT_FindCertIssuer
+	 * this function is not using the authCertIssuer field as a fallback
+	 * if authority key id does not exist
+	 */
 	if (issuerID) {
 	    certs = filter_subject_certs_for_id(certs, issuerID);
 	}
@@ -456,10 +469,9 @@ find_cert_issuer (
     return issuer;
 }
 
-/* XXX review based on CERT_FindCertIssuer
- * this function is not using the authCertIssuer field as a fallback
- * if authority key id does not exist
- */
+/* This function returns the built chain, as far as it gets,
+** even if/when it fails to find an issuer, and returns PR_FAILURE
+*/
 NSS_IMPLEMENT NSSCertificate **
 nssCertificate_BuildChain (
   NSSCertificate *c,
@@ -469,72 +481,67 @@ nssCertificate_BuildChain (
   NSSCertificate **rvOpt,
   PRUint32 rvLimit,
   NSSArena *arenaOpt,
-  PRStatus *statusOpt
+  PRStatus *statusOpt,
+  NSSTrustDomain *td,
+  NSSCryptoContext *cc 
 )
 {
-    NSSCertificate **rvChain;
-#ifdef NSS_3_4_CODE
-    NSSCertificate *cp;
-    CERTCertificate *cCert;
-#endif
+    NSSCertificate **rvChain = NULL;
     NSSUsage issuerUsage = *usage;
-    NSSTrustDomain *td;
-    nssPKIObjectCollection *collection;
+    nssPKIObjectCollection *collection = NULL;
+    PRUint32  rvCount = 0;
+    PRStatus  st;
+    PRStatus  ret = PR_SUCCESS;
 
-    td = NSSCertificate_GetTrustDomain(c);
-#ifdef NSS_3_4_CODE
-    if (!td) {
-	td = STAN_GetDefaultTrustDomain();
+    if (!c || !cc ||
+        (!td && (td = NSSCertificate_GetTrustDomain(c)) == NULL)) {
+	goto loser;
     }
     /* bump the usage up to CA level */
     issuerUsage.nss3lookingForCA = PR_TRUE;
-#endif
-    if (statusOpt) *statusOpt = PR_SUCCESS;
     collection = nssCertificateCollection_Create(td, NULL);
-    if (!collection) {
-	if (statusOpt) *statusOpt = PR_FAILURE;
-	return (NSSCertificate **)NULL;
-    }
-    nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
-    if (rvLimit == 1) {
-	goto finish;
-    }
-    /* XXX This breaks code for which NSS_3_4_CODE is not defined (pure
-     *     4.0 builds).  That won't affect the tip.  But be careful
-     *     when merging 4.0!!!
-     */
-    while (c != (NSSCertificate *)NULL) {
-#ifdef NSS_3_4_CODE
-	cCert = STAN_GetCERTCertificate(c);
+    if (!collection)
+	goto loser;
+    st = nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
+    if (st != PR_SUCCESS)
+    	goto loser;
+    for (rvCount = 1; (!rvLimit || rvCount < rvLimit); ++rvCount) {
+	CERTCertificate *cCert = STAN_GetCERTCertificate(c);
 	if (cCert->isRoot) {
 	    /* not including the issuer of the self-signed cert, which is,
 	     * of course, itself
 	     */
 	    break;
 	}
-	cp = c;
-#endif
-	c = find_cert_issuer(c, timeOpt, &issuerUsage, policiesOpt);
-	if (c) {
-	    nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
-	    nssCertificate_Destroy(c); /* collection has it */
-	    if (rvLimit > 0 &&
-	        nssPKIObjectCollection_Count(collection) == rvLimit) 
-	    {
-		break;
-	    }
-	} else {
-	    nss_SetError(NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND);
-	    if (statusOpt) *statusOpt = PR_FAILURE;
+	c = find_cert_issuer(c, timeOpt, &issuerUsage, policiesOpt, td, cc);
+	if (!c) {
+	    ret = PR_FAILURE;
 	    break;
 	}
+	st = nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
+	nssCertificate_Destroy(c); /* collection has it */
+	if (st != PR_SUCCESS)
+	    goto loser;
     }
-finish:
     rvChain = nssPKIObjectCollection_GetCertificates(collection, 
                                                      rvOpt, 
                                                      rvLimit, 
                                                      arenaOpt);
-    nssPKIObjectCollection_Destroy(collection);
+    if (rvChain) {
+	nssPKIObjectCollection_Destroy(collection);
+	if (statusOpt) 
+	    *statusOpt = ret;
+	if (ret != PR_SUCCESS)
+	    nss_SetError(NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND);
+	return rvChain;
+    }
+
+loser:
+    if (collection)
+	nssPKIObjectCollection_Destroy(collection);
+    if (statusOpt) 
+	*statusOpt = PR_FAILURE;
+    nss_SetError(NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND);
     return rvChain;
 }
 
@@ -547,11 +554,14 @@ NSSCertificate_BuildChain (
   NSSCertificate **rvOpt,
   PRUint32 rvLimit, /* zero for no limit */
   NSSArena *arenaOpt,
-  PRStatus *statusOpt
+  PRStatus *statusOpt,
+  NSSTrustDomain *td,
+  NSSCryptoContext *cc 
 )
 {
     return nssCertificate_BuildChain(c, timeOpt, usage, policiesOpt,
-                                     rvOpt, rvLimit, arenaOpt, statusOpt);
+                                     rvOpt, rvLimit, arenaOpt, statusOpt,
+				     td, cc);
 }
 
 NSS_IMPLEMENT NSSCryptoContext *
@@ -756,6 +766,26 @@ NSSCertificate_IsPrivateKeyAvailable (
     return isUser;
 }
 
+/* sort the subject cert list from newest to oldest */
+PRIntn
+nssCertificate_SubjectListSort (
+  void *v1,
+  void *v2
+)
+{
+    NSSCertificate *c1 = (NSSCertificate *)v1;
+    NSSCertificate *c2 = (NSSCertificate *)v2;
+    nssDecodedCert *dc1 = nssCertificate_GetDecoding(c1);
+    nssDecodedCert *dc2 = nssCertificate_GetDecoding(c2);
+    if (!dc1) {
+	return dc2 ? 1 : 0;
+    } else if (!dc2) {
+	return -1;
+    } else {
+	return dc1->isNewerThan(dc1, dc2) ? -1 : 1;
+    }
+}
+
 NSS_IMPLEMENT PRBool
 NSSUserCertificate_IsStillPresent (
   NSSUserCertificate *uc,
@@ -849,83 +879,6 @@ NSSUserCertificate_DeriveSymmetricKey (
     return NULL;
 }
 
-NSS_IMPLEMENT void 
-nssBestCertificate_SetArgs (
-  nssBestCertificateCB *best,
-  NSSTime *timeOpt,
-  NSSUsage *usage,
-  NSSPolicies *policies
-)
-{
-    if (timeOpt) {
-	best->time = timeOpt;
-    } else {
-	NSSTime_Now(&best->sTime);
-	best->time = &best->sTime;
-    }
-    best->usage = usage;
-    best->policies = policies;
-    best->cert = NULL;
-}
-
-NSS_IMPLEMENT PRStatus 
-nssBestCertificate_Callback (
-  NSSCertificate *c, 
-  void *arg
-)
-{
-    nssBestCertificateCB *best = (nssBestCertificateCB *)arg;
-    nssDecodedCert *dc, *bestdc;
-    dc = nssCertificate_GetDecoding(c);
-    if (!best->cert) {
-	/* usage */
-	if (best->usage->anyUsage) {
-	    best->cert = nssCertificate_AddRef(c);
-	} else {
-#ifdef NSS_3_4_CODE
-	    /* For this to work in NSS 3.4, we have to go out and fill in
-	     * all of the CERTCertificate fields.  Why?  Because the
-	     * matchUsage function calls CERT_IsCACert, which needs to know
-	     * what the trust values are for the cert.
-	     * Ignore the returned pointer, the refcount is in c anyway.
-	     */
-	    if (STAN_GetCERTCertificate(c) == NULL) {
-		return PR_FAILURE;
-	    }
-#endif
-	    if (dc->matchUsage(dc, best->usage)) {
-		best->cert = nssCertificate_AddRef(c);
-	    }
-	}
-	return PR_SUCCESS;
-    }
-    bestdc = nssCertificate_GetDecoding(best->cert);
-    /* time */
-    if (bestdc->isValidAtTime(bestdc, best->time)) {
-	/* The current best cert is valid at time */
-	if (!dc->isValidAtTime(dc, best->time)) {
-	    /* If the new cert isn't valid at time, it's not better */
-	    return PR_SUCCESS;
-	}
-    } else {
-	/* The current best cert is not valid at time */
-	if (dc->isValidAtTime(dc, best->time)) {
-	    /* If the new cert is valid at time, it's better */
-	    NSSCertificate_Destroy(best->cert);
-	    best->cert = nssCertificate_AddRef(c);
-	    return PR_SUCCESS;
-	}
-    }
-    /* either they are both valid at time, or neither valid; take the newer */
-    /* XXX later -- defer to policies */
-    if (!bestdc->isNewerThan(bestdc, dc)) {
-	NSSCertificate_Destroy(best->cert);
-	best->cert = nssCertificate_AddRef(c);
-    }
-    /* policies */
-    return PR_SUCCESS;
-}
-
 NSS_IMPLEMENT nssSMIMEProfile *
 nssSMIMEProfile_Create (
   NSSCertificate *cert,
@@ -942,7 +895,7 @@ nssSMIMEProfile_Create (
     if (!arena) {
 	return NULL;
     }
-    object = nssPKIObject_Create(arena, NULL, td, cc);
+    object = nssPKIObject_Create(arena, NULL, td, cc, nssPKILock);
     if (!object) {
 	goto loser;
     }
@@ -962,7 +915,8 @@ nssSMIMEProfile_Create (
     }
     return rvProfile;
 loser:
-    nssPKIObject_Destroy(object);
+    if (object) nssPKIObject_Destroy(object);
+    else if (arena)  nssArena_Destroy(arena);
     return (nssSMIMEProfile *)NULL;
 }
 
@@ -978,6 +932,9 @@ nssCertificateList_DoCallback (
     NSSCertificate *cert;
     PRStatus nssrv;
     certs = nssList_CreateIterator(certList);
+    if (!certs) {
+        return PR_FAILURE;
+    }
     for (cert  = (NSSCertificate *)nssListIterator_Start(certs);
          cert != (NSSCertificate *)NULL;
          cert  = (NSSCertificate *)nssListIterator_Next(certs))
@@ -1019,6 +976,8 @@ nssTrust_Create (
     nssCryptokiObject *instance;
     nssTrustLevel serverAuth, clientAuth, codeSigning, emailProtection;
     SECStatus rv; /* Should be stan flavor */
+    PRBool stepUp;
+
     lastTrustOrder = 1<<16; /* just make it big */
     PR_ASSERT(object->instances != NULL && object->numInstances > 0);
     rvt = nss_ZNEW(object->arena, NSSTrust);
@@ -1035,7 +994,7 @@ nssTrust_Create (
     sha1_hash.data = sha1_hashin;
     sha1_hash.size = sizeof (sha1_hashin);
     /* trust has to peek into the base object members */
-    PZ_Lock(object->lock);
+    nssPKIObject_Lock(object);
     for (i=0; i<object->numInstances; i++) {
 	instance = object->instances[i];
 	myTrustOrder = nssToken_GetTrustOrder(instance->token);
@@ -1044,13 +1003,14 @@ nssTrust_Create (
 	                                        &serverAuth,
 	                                        &clientAuth,
 	                                        &codeSigning,
-	                                        &emailProtection);
+	                                        &emailProtection,
+	                                        &stepUp);
 	if (status != PR_SUCCESS) {
-	    PZ_Unlock(object->lock);
+	    nssPKIObject_Unlock(object);
 	    return (NSSTrust *)NULL;
 	}
 	if (PORT_Memcmp(sha1_hashin,sha1_hashcmp,SHA1_LENGTH) != 0) {
-	    PZ_Unlock(object->lock);
+	    nssPKIObject_Unlock(object);
 	    return (NSSTrust *)NULL;
 	}
 	if (rvt->serverAuth == nssTrustLevel_Unknown ||
@@ -1073,9 +1033,10 @@ nssTrust_Create (
 	{
 	    rvt->codeSigning = codeSigning;
 	}
+	rvt->stepUpApproved = stepUp;
 	lastTrustOrder = myTrustOrder;
     }
-    PZ_Unlock(object->lock);
+    nssPKIObject_Unlock(object);
     return rvt;
 }
 
@@ -1188,7 +1149,7 @@ nssCRL_GetEncoding (
   NSSCRL *crl
 )
 {
-    if (crl->encoding.data != NULL && crl->encoding.size > 0) {
+    if (crl && crl->encoding.data != NULL && crl->encoding.size > 0) {
 	return &crl->encoding;
     } else {
 	return (NSSDER *)NULL;
