@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *  Darin Fisher <darin@meer.net>
+ *  Robert Strong <robert.bugzilla@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -47,6 +48,41 @@
  *  LWS      = 1*( " " | "\t" )
  */
 
+#if defined(XP_WIN)
+# include <windows.h>
+# include <direct.h>
+# include <io.h>
+# define F_OK 00
+# define W_OK 02
+# define R_OK 04
+# define access _access
+# define putenv _putenv
+# define snprintf _snprintf
+# define fchmod(a,b)
+# define mkdir(path, perms) _mkdir(path)
+
+# define NS_T(str) L ## str
+# define NS_tsnprintf _snwprintf
+# define NS_tstrrchr wcsrchr
+# define NS_tchdir _wchdir
+# define NS_tremove _wremove
+# define NS_topen _wopen
+# define NS_tfopen _wfopen
+# define NS_tatoi _wtoi64
+#else
+# include <sys/wait.h>
+# include <unistd.h>
+
+# define NS_T(str) str
+# define NS_tsnprintf snprintf
+# define NS_tstrrchr strrchr
+# define NS_tchdir chdir
+# define NS_tremove remove
+# define NS_topen open
+# define NS_tfopen fopen
+# define NS_tatoi atoi
+#endif
+
 #include "bspatch.h"
 #include "progressui.h"
 #include "archivereader.h"
@@ -64,24 +100,6 @@
 #include <limits.h>
 #include <errno.h>
 
-#if defined(XP_WIN)
-# include <windows.h>
-# include <direct.h>
-# include <io.h>
-# define F_OK 00
-# define W_OK 02
-# define R_OK 04
-# define access _access
-# define snprintf _snprintf
-# define putenv _putenv
-# define fchmod(a,b)
-# define mkdir(path, perm) _mkdir(path)
-# define chdir(path) _chdir(path)
-#else
-# include <sys/wait.h>
-# include <unistd.h>
-#endif
-
 #if defined(XP_MACOSX)
 // This function is defined in launchchild_osx.mm
 void LaunchChild(int argc, char **argv);
@@ -100,7 +118,9 @@ void LaunchChild(int argc, char **argv);
 #endif
 
 #ifndef MAXPATHLEN
-# ifdef MAX_PATH
+# ifdef PATH_MAX
+#  define MAXPATHLEN PATH_MAX
+# elif defined(_MAX_PATH)
 #  define MAXPATHLEN MAX_PATH
 # elif defined(_MAX_PATH)
 #  define MAXPATHLEN _MAX_PATH
@@ -121,7 +141,11 @@ void LaunchChild(int argc, char **argv);
 
 // This variable lives in libbz2.  It's declared in bzlib_private.h, so we just
 // declare it here to avoid including that entire header file.
+#if (__GNUC__ >= 4) || (__GNUC__ == 3 && __GNUC_MINOR__ >= 3)
+extern "C"  __attribute__((visibility("default"))) unsigned int BZ2_crc32Table[256];
+#else
 extern "C" unsigned int BZ2_crc32Table[256];
+#endif
 
 static unsigned int
 crc32(const unsigned char *buf, unsigned int len)
@@ -245,11 +269,10 @@ public:
     return 0;
   }
 private:
-  static unsigned ThreadMain(void *p)
+  static void ThreadMain(void *p)
   {
     Thread *self = (Thread *) p;
     self->mThreadFunc(self->mThreadParam);
-    return 0;
   }
   int        mThread;
   ThreadFunc mThreadFunc;
@@ -262,8 +285,11 @@ private:
 
 //-----------------------------------------------------------------------------
 
-static char* gSourcePath;
+static NS_tchar* gSourcePath;
 static ArchiveReader gArchiveReader;
+#ifdef XP_WIN
+static bool gSucceeded = FALSE;
+#endif
 
 static const char kWhitespace[] = " \t";
 static const char kNL[] = "\r\n";
@@ -279,10 +305,10 @@ static void LogInit()
   if (gLogFP)
     return;
 
-  char logFile[MAXPATHLEN];
-  snprintf(logFile, MAXPATHLEN, "%s/update.log", gSourcePath);
+  NS_tchar logFile[MAXPATHLEN];
+  NS_tsnprintf(logFile, MAXPATHLEN, NS_T("%s/update.log"), gSourcePath);
 
-  gLogFP = fopen(logFile, "w");
+  gLogFP = NS_tfopen(logFile, NS_T("w"));
 }
 
 static void LogFinish()
@@ -357,7 +383,7 @@ mstrtok(const char *delims, char **str)
 static void ensure_write_permissions(const char *path)
 {
 #ifdef XP_WIN
-  (void)_chmod(path, _S_IREAD | _S_IWRITE);
+  (void) chmod(path, _S_IREAD | _S_IWRITE);
 #else
   struct stat fs;
   if (!stat(path, &fs) && !(fs.st_mode & S_IWUSR)) {
@@ -522,7 +548,7 @@ public:
 
   // Perform the operation.  Return OK to indicate success.  After all actions
   // have been executed, Finish will be called.  A requirement of Execute is
-  // that it's operation be reversable from Finish.
+  // that its operation be reversable from Finish.
   virtual int Execute() = 0;
   
   // Finish is called after execution of all actions.  If status is OK, then
@@ -739,9 +765,11 @@ PatchFile::~PatchFile()
     close(pfd);
 
   // delete the temporary patch file
-  char spath[MAXPATHLEN];
-  snprintf(spath, MAXPATHLEN, "%s/%d.patch", gSourcePath, mPatchIndex);
-  ensure_remove(spath);
+  NS_tchar spath[MAXPATHLEN];
+  NS_tsnprintf(spath, MAXPATHLEN, NS_T("%s/%d.patch"),
+               gSourcePath, mPatchIndex);
+
+  NS_tremove(spath);
 
   free(buf);
 }
@@ -816,12 +844,18 @@ PatchFile::Prepare()
   // extract the patch to a temporary file
   mPatchIndex = sPatchIndex++;
 
-  char spath[MAXPATHLEN];
-  snprintf(spath, MAXPATHLEN, "%s/%d.patch", gSourcePath, mPatchIndex);
+  NS_tchar spath[MAXPATHLEN];
+  NS_tsnprintf(spath, MAXPATHLEN, NS_T("%s/%d.patch"),
+               gSourcePath, mPatchIndex);
 
-  ensure_remove(spath);
+  NS_tremove(spath);
 
-  int rv = gArchiveReader.ExtractFile(mPatchFile, spath);
+  FILE *fp = NS_tfopen(spath, NS_T("wb"));
+  if (!fp)
+    return WRITE_ERROR;
+
+  int rv = gArchiveReader.ExtractFileToStream(mPatchFile, fp);
+  fclose(fp);
   if (rv)
     return rv;
 
@@ -829,7 +863,7 @@ PatchFile::Prepare()
   //          no need to open all of the patch files and read all of 
   //          the source files before applying any patches.
 
-  pfd = open(spath, O_RDONLY | _O_BINARY);
+  pfd = NS_topen(spath, O_RDONLY | _O_BINARY);
   if (pfd < 0)
     return READ_ERROR;
 
@@ -1007,23 +1041,89 @@ PatchIfFile::Finish(int status)
 
 #ifdef XP_WIN
 #include "nsWindowsRestart.cpp"
+
+static void
+copyASCIItoWCHAR(WCHAR *dest, const char *src)
+{
+  while (*src) {
+    *dest = *src;
+    ++src; ++dest;
+  }
+}
+
+static void
+LaunchWinPostProcess(const WCHAR *appExe)
+{
+  // Launch helper.exe to perform post processing (e.g. registry and log file
+  // modifications) for the update.
+  WCHAR inifile[MAXPATHLEN];
+  wcscpy(inifile, appExe);
+
+  WCHAR *slash = wcsrchr(inifile, '\\');
+  if (!slash)
+    return;
+
+  wcscpy(slash + 1, L"updater.ini");
+
+  WCHAR exefile[MAXPATHLEN];
+  WCHAR exearg[MAXPATHLEN];
+
+  if (!GetPrivateProfileStringW(L"PostUpdateWin", L"ExeRelPath", NULL, exefile,
+                                MAXPATHLEN, inifile))
+    return;
+
+  if (!GetPrivateProfileStringW(L"PostUpdateWin", L"ExeArg", NULL, exearg,
+                                MAXPATHLEN, inifile))
+    return;
+
+  WCHAR exefullpath[MAXPATHLEN];
+  wcscpy(exefullpath, appExe);
+
+  slash = wcsrchr(exefullpath, '\\');
+  wcscpy(slash + 1, exefile);
+
+  WCHAR dlogFile[MAXPATHLEN];
+  wcscpy(dlogFile, exefullpath);
+
+  slash = wcsrchr(dlogFile, '\\');
+  wcscpy(slash + 1, L"uninstall.update");
+
+  WCHAR slogFile[MAXPATHLEN];
+  _snwprintf(slogFile, MAXPATHLEN, L"%s/update.log", gSourcePath);
+
+  // We want to launch the post update helper app to update the Windows
+  // registry even if there is a failure with removing the uninstall.update
+  // file or copying the update.log file.
+  NS_tremove(dlogFile);
+  CopyFile(slogFile, dlogFile, FALSE);
+
+  static int    argc = 2;
+  static WCHAR* argv[3] = {
+    L"argv0ignoredbywinlaunchchild",
+    exearg,
+    L"\0"
+  };
+
+  WinLaunchChild(exefullpath, argc, argv, 0);
+  free(argv);
+}
 #endif
 
 static void
-LaunchCallbackApp(const char *workingDir, int argc, char **argv)
+LaunchCallbackApp(const NS_tchar *workingDir, int argc, NS_tchar **argv)
 {
   putenv("NO_EM_RESTART=");
   putenv("MOZ_LAUNCHED_CHILD=1");
 
   // Run from the specified working directory (see bug 312360).
-  chdir(workingDir);
+  NS_tchdir(workingDir);
 
 #if defined(USE_EXECV)
   execv(argv[0], argv);
 #elif defined(XP_MACOSX)
   LaunchChild(argc, argv);
 #elif defined(XP_WIN)
-  WinLaunchChild(argv[0], argc, argv, -1);
+  WinLaunchChild(argv[0], argc, argv, 0);
 #else
 # warning "Need implementaton of LaunchCallbackApp"
 #endif
@@ -1034,10 +1134,10 @@ WriteStatusFile(int status)
 {
   // This is how we communicate our completion status to the main application.
 
-  char filename[MAXPATHLEN];
-  snprintf(filename, MAXPATHLEN, "%s/update.status", gSourcePath);
+  NS_tchar filename[MAXPATHLEN];
+  NS_tsnprintf(filename, MAXPATHLEN, NS_T("%s/update.status"), gSourcePath);
 
-  AutoFD fd = ensure_open(filename, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, 0644);
+  AutoFD fd = NS_topen(filename, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, 0644);
   if (fd < 0)
     return;
 
@@ -1058,8 +1158,8 @@ UpdateThreadFunc(void *param)
 {
   // open ZIP archive and process...
 
-  char dataFile[MAXPATHLEN];
-  snprintf(dataFile, MAXPATHLEN, "%s/update.mar", gSourcePath);
+  NS_tchar dataFile[MAXPATHLEN];
+  NS_tsnprintf(dataFile, MAXPATHLEN, NS_T("%s/update.mar"), gSourcePath);
 
   int rv = gArchiveReader.Open(dataFile);
   if (rv == OK) {
@@ -1077,7 +1177,7 @@ UpdateThreadFunc(void *param)
   QuitProgressUI();
 }
 
-int main(int argc, char **argv)
+int NS_main(int argc, NS_tchar **argv)
 {
   InitProgressUI(&argc, &argv);
 
@@ -1093,7 +1193,7 @@ int main(int argc, char **argv)
     return 1;
   }
 
-  int pid = atoi(argv[2]);
+  int pid = NS_tatoi(argv[2]);
   if (pid) {
 #ifdef XP_WIN
     HANDLE parent = OpenProcess(SYNCHRONIZE, FALSE, (DWORD) pid);
@@ -1115,6 +1215,92 @@ int main(int argc, char **argv)
 #endif
   }
 
+#ifdef XP_WIN
+  // Launch a second instance of the updater with the runas verb on Windows
+  // when write access is denied to the installation directory.
+  HANDLE updateLockFileHandle;
+  NS_tchar elevatedLockFilePath[MAXPATHLEN];
+  if (argc > 4) {
+    NS_tchar updateLockFilePath[MAXPATHLEN];
+    NS_tsnprintf(updateLockFilePath, MAXPATHLEN,
+                 NS_T("%s.update_in_progress.lock"), argv[4]);
+
+    // The update_in_progress.lock file should only exist during an update. In
+    // case it exists attempt to remove it and exit if that fails to prevent
+    // simultaneous updates occurring.
+    if (!_waccess(updateLockFilePath, F_OK) &&
+        NS_tremove(updateLockFilePath) != 0) {
+      fprintf(stderr, "Update already in progress! Exiting\n");
+      return 1;
+    }
+
+    updateLockFileHandle = CreateFileW(updateLockFilePath,
+                                       GENERIC_READ | GENERIC_WRITE,
+                                       0,
+                                       NULL,
+                                       OPEN_ALWAYS,
+                                       FILE_FLAG_DELETE_ON_CLOSE,
+                                       NULL);
+
+    NS_tsnprintf(elevatedLockFilePath, MAXPATHLEN,
+                 NS_T("%s/update_elevated.lock"), argv[1]);
+
+    if (updateLockFileHandle == INVALID_HANDLE_VALUE) {
+      if (!_waccess(elevatedLockFilePath, F_OK) &&
+          NS_tremove(elevatedLockFilePath) != 0) {
+        fprintf(stderr, "Update already elevated! Exiting\n");
+        return 1;
+      }
+
+      HANDLE elevatedFileHandle;
+      elevatedFileHandle = CreateFileW(elevatedLockFilePath,
+                                       GENERIC_READ | GENERIC_WRITE,
+                                       0,
+                                       NULL,
+                                       OPEN_ALWAYS,
+                                       FILE_FLAG_DELETE_ON_CLOSE,
+                                       NULL);
+
+      if (elevatedFileHandle == INVALID_HANDLE_VALUE) {
+        fprintf(stderr, "Unable to create elevated lock file! Exiting\n");
+        return 1;
+      }
+
+      PRUnichar *cmdLine = MakeCommandLine(argc - 1, argv + 1);
+      if (!cmdLine) {
+        CloseHandle(elevatedFileHandle);
+        return 1;
+      }
+
+      SHELLEXECUTEINFO sinfo;
+      memset(&sinfo, 0, sizeof(SHELLEXECUTEINFO));
+      sinfo.cbSize       = sizeof(SHELLEXECUTEINFO);
+      sinfo.fMask        = SEE_MASK_FLAG_DDEWAIT |
+                           SEE_MASK_FLAG_NO_UI |
+                           SEE_MASK_NOCLOSEPROCESS;
+      sinfo.hwnd         = NULL;
+      sinfo.lpFile       = argv[0];
+      sinfo.lpParameters = cmdLine;
+      sinfo.lpVerb       = L"runas";
+      sinfo.nShow        = SW_SHOWNORMAL;
+
+      BOOL result = ShellExecuteEx(&sinfo);
+      free(cmdLine);
+
+      if (result) {
+        WaitForSingleObject(sinfo.hProcess, INFINITE);
+        CloseHandle(sinfo.hProcess);
+      }
+
+      if (argc > 4)
+        LaunchCallbackApp(argv[3], argc - 4, argv + 4);
+
+      CloseHandle(elevatedFileHandle);
+      return 0;
+    }
+  }
+#endif
+
   gSourcePath = argv[1];
 
   LogInit();
@@ -1129,6 +1315,20 @@ int main(int argc, char **argv)
 
   LogFinish();
 
+#ifdef XP_WIN
+  if (gSucceeded && argc > 4)
+    LaunchWinPostProcess(argv[4]);
+
+  if (argc > 4) {
+    CloseHandle(updateLockFileHandle);
+    // If elevated return early and let the process that launched this process
+    // launch the callback application.
+    if (!_waccess(elevatedLockFilePath, F_OK) &&
+        NS_tremove(elevatedLockFilePath) != 0)
+      return 0;
+  }
+#endif
+
   // The callback to execute is given as the last N arguments of our command
   // line.  The first of those arguments specifies the working directory for
   // the callback.
@@ -1234,20 +1434,30 @@ ActionList::Finish(int status)
     a = a->mNext;
   }
 
+#ifdef XP_WIN
+  if (status == OK)
+    gSucceeded = TRUE;
+#endif
+
   UpdateProgressUI(100.0f);
 }
 
 int DoUpdate()
 {
-  char manifest[MAXPATHLEN];
-  snprintf(manifest, MAXPATHLEN, "%s/update.manifest", gSourcePath);
+  NS_tchar manifest[MAXPATHLEN];
+  NS_tsnprintf(manifest, MAXPATHLEN, NS_T("%s/update.manifest"), gSourcePath);
 
   // extract the manifest
-  int rv = gArchiveReader.ExtractFile("update.manifest", manifest);
+  FILE *fp = NS_tfopen(manifest, NS_T("wb"));
+  if (!fp)
+    return READ_ERROR;
+
+  int rv = gArchiveReader.ExtractFileToStream("update.manifest", fp);
+  fclose(fp);
   if (rv)
     return rv;
 
-  AutoFD mfd = open(manifest, O_RDONLY | _O_BINARY);
+  AutoFD mfd = NS_topen(manifest, O_RDONLY | _O_BINARY);
   if (mfd < 0)
     return READ_ERROR;
 
@@ -1327,13 +1537,3 @@ int DoUpdate()
   list.Finish(rv);
   return rv;
 }
-
-#if defined(XP_WIN) && !defined(DEBUG) && !defined(__GNUC__)
-// We need WinMain in order to not be a console app.  This function is unused
-// if we are a console application.
-int WINAPI WinMain( HINSTANCE, HINSTANCE, LPSTR args, int )
-{
-  // Do the real work.
-  return main(__argc, __argv);
-}
-#endif
